@@ -1,4 +1,4 @@
-(* Lightweight thread library for Objective Caml
+(* Lightweight thread library for OCaml
  * http://www.ocsigen.org/lwt
  * Program discover
  * Copyright (C) 2010 Jérémie Dimino
@@ -39,9 +39,10 @@ open Printf
 let ( // ) = Filename.concat
 
 let default_search_paths =
-  List.map (fun dir -> (dir ^ "/include", dir ^ "/lib")) [
+  List.map (fun dir -> (dir // "include", dir // "lib")) [
     "/usr";
     "/usr/local";
+    "/usr/pkg";
     "/opt";
     "/opt/local";
     "/sw";
@@ -61,6 +62,16 @@ let split_path str =
   in
   aux 0
 
+let rec replace_last path ~patt ~repl =
+  let comp = Filename.basename path
+  and parent = Filename.dirname path in
+  if comp = patt then
+    parent // repl
+  else if parent = path then
+    path
+  else
+    (replace_last parent ~patt ~repl) // comp
+
 let search_paths =
   let get var f =
     try
@@ -69,8 +80,8 @@ let search_paths =
       []
   in
   List.flatten [
-    get "C_INCLUDE_PATH" (fun dir -> (dir, dir // ".." // "lib"));
-    get "LIBRARY_PATH" (fun dir -> (dir // ".." // "include", dir));
+    get "C_INCLUDE_PATH" (fun dir -> (dir, replace_last dir ~patt:"include" ~repl:"lib"));
+    get "LIBRARY_PATH" (fun dir -> (replace_last dir ~patt:"lib" ~repl:"include", dir));
     default_search_paths;
   ]
 
@@ -87,7 +98,7 @@ let pthread_code = "
 #include <caml/mlvalues.h>
 #include <pthread.h>
 
-CAMLprim value lwt_test()
+CAMLprim value lwt_test(value Unit)
 {
   pthread_create(0, 0, 0, 0);
   return Val_unit;
@@ -98,7 +109,7 @@ let libev_code = "
 #include <caml/mlvalues.h>
 #include <ev.h>
 
-CAMLprim value lwt_test()
+CAMLprim value lwt_test(value Unit)
 {
   ev_default_loop(0);
   return Val_unit;
@@ -110,7 +121,7 @@ let fd_passing_code = "
 #include <sys/types.h>
 #include <sys/socket.h>
 
-CAMLprim value lwt_test()
+CAMLprim value lwt_test(value Unit)
 {
   struct msghdr msg;
   msg.msg_controllen = 0;
@@ -124,7 +135,7 @@ let getcpu_code = "
 #define _GNU_SOURCE
 #include <sched.h>
 
-CAMLprim value lwt_test()
+CAMLprim value lwt_test(value Unit)
 {
   sched_getcpu();
   return Val_unit;
@@ -136,7 +147,7 @@ let affinity_code = "
 #define _GNU_SOURCE
 #include <sched.h>
 
-CAMLprim value lwt_test()
+CAMLprim value lwt_test(value Unit)
 {
   sched_getaffinity(0, 0, 0);
   return Val_unit;
@@ -147,7 +158,7 @@ let eventfd_code = "
 #include <caml/mlvalues.h>
 #include <sys/eventfd.h>
 
-CAMLprim value lwt_test()
+CAMLprim value lwt_test(value Unit)
 {
   eventfd(0, 0);
   return Val_unit;
@@ -160,7 +171,7 @@ let get_credentials_code struct_name = "
 #include <sys/types.h>
 #include <sys/socket.h>
 
-CAMLprim value lwt_test()
+CAMLprim value lwt_test(value Unit)
 {
   struct " ^ struct_name ^ " cred;
   socklen_t cred_len = sizeof(cred);
@@ -174,7 +185,7 @@ let get_peereid_code = "
 #include <sys/types.h>
 #include <unistd.h>
 
-CAMLprim value lwt_test()
+CAMLprim value lwt_test(value Unit)
 {
   uid_t euid;
   gid_t egid;
@@ -187,7 +198,7 @@ let fdatasync_code = "
 #include <caml/mlvalues.h>
 #include <sys/unistd.h>
 
-CAMLprim value lwt_test()
+CAMLprim value lwt_test(value Unit)
 {
   fdatasync(0);
   return Val_unit;
@@ -198,7 +209,7 @@ let glib_code = "
 #include <caml/mlvalues.h>
 #include <glib.h>
 
-CAMLprim value lwt_test()
+CAMLprim value lwt_test(value Unit)
 {
   g_main_context_dispatch(0);
   return Val_unit;
@@ -206,15 +217,63 @@ CAMLprim value lwt_test()
 "
 
 let netdb_reentrant_code = "
+#define _POSIX_PTHREAD_SEMANTICS
 #include <caml/mlvalues.h>
 #include <netdb.h>
+#include <stddef.h>
 
-CAMLprim value lwt_test()
+CAMLprim value lwt_test(value Unit)
 {
-  getprotobyname_r(0, 0, 0, 0, 0);
-  getprotobynumber_r(0, 0, 0, 0, 0);
-  getservbyname_r(0, 0, 0, 0, 0, 0);
-  getservbyport_r(0, 0, 0, 0, 0, 0);
+  struct hostent *he;
+  struct servent *se;
+  he = gethostbyname_r((const char *)NULL, (struct hostent *)NULL,(char *)NULL, (int)0, (struct hostent **)NULL, (int *)NULL);
+  he = gethostbyaddr_r((const char *)NULL, (int)0, (int)0,(struct hostent *)NULL, (char *)NULL, (int)0, (struct hostent **)NULL,(int *)NULL);
+  se = getservbyname_r((const char *)NULL, (const char *)NULL,(struct servent *)NULL, (char *)NULL, (int)0, (struct servent **)NULL);
+  se = getservbyport_r((int)0, (const char *)NULL,(struct servent *)NULL, (char *)NULL, (int)0, (struct servent **)NULL);
+  return Val_unit;
+}
+"
+
+let hostent_reentrant_code = "
+#define _GNU_SOURCE
+#include <stddef.h>
+#include <caml/mlvalues.h>
+#include <caml/config.h>
+/* Helper functions for not re-entrant functions */
+#if !defined(HAS_GETHOSTBYADDR_R) || (HAS_GETHOSTBYADDR_R != 7 && HAS_GETHOSTBYADDR_R != 8)
+#define NON_R_GETHOSTBYADDR 1
+#endif
+
+#if !defined(HAS_GETHOSTBYNAME_R) || (HAS_GETHOSTBYNAME_R != 5 && HAS_GETHOSTBYNAME_R != 6)
+#define NON_R_GETHOSTBYNAME 1
+#endif
+
+CAMLprim value lwt_test(value u)
+{
+  (void)u;
+#if defined(NON_R_GETHOSTBYNAME) || defined(NON_R_GETHOSTBYNAME)
+#error \"not available\"
+#else
+  return Val_unit;
+#endif
+}
+"
+
+let struct_ns_code conversion = "
+#define _GNU_SOURCE
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
+#include <caml/mlvalues.h>
+
+#define NANOSEC" ^ conversion ^ "
+
+CAMLprim value lwt_test() {
+  struct stat *buf;
+  double a, m, c;
+  a = (double)NANOSEC(buf, a);
+  m = (double)NANOSEC(buf, m);
+  c = (double)NANOSEC(buf, c);
   return Val_unit;
 }
 "
@@ -223,7 +282,7 @@ CAMLprim value lwt_test()
    | Compilation                                                     |
    +-----------------------------------------------------------------+ *)
 
-let ocamlc = ref "ocamlc"
+let ocamlc = ref "ocamlfind ocamlc"
 let ext_obj = ref ".o"
 let exec_name = ref "a.out"
 let use_libev = ref true
@@ -231,7 +290,18 @@ let use_glib = ref false
 let use_pthread = ref true
 let use_unix = ref true
 let os_type = ref "Unix"
+let android_target = ref false
 let ccomp_type = ref "cc"
+let libev_default = ref true
+let debug = ref (try Sys.getenv "DEBUG" = "y" with Not_found -> false)
+
+let dprintf fmt =
+  if !debug then
+    (
+      eprintf "DBG: ";
+      kfprintf (fun oc -> fprintf oc "\n%!") stderr fmt
+    )
+  else ifprintf stderr fmt
 
 let log_file = ref ""
 let caml_file = ref ""
@@ -249,29 +319,45 @@ let search_header header =
   in
   loop search_paths
 
+(* CFLAGS should not be passed to the linker. *)
 let compile (opt, lib) stub_file =
-  ksprintf
-    Sys.command
-    "%s -custom %s %s %s %s > %s 2>&1"
+  let cmd fmt = ksprintf (fun s ->
+    dprintf "RUN: %s" s;
+    Sys.command s = 0) fmt in
+  let obj_file = Filename.chop_suffix stub_file ".c" ^ !ext_obj
+  in
+  (* First compile the .c file using -ocamlc and CFLAGS (opt) *)
+  (cmd
+    "%s -c %s %s -ccopt -o -ccopt %s >> %s 2>&1"
     !ocamlc
-    (String.concat " " (List.map (sprintf "-ccopt %s") opt))
+    (String.concat " " (List.map (fun x -> "-ccopt " ^ x) (List.map Filename.quote opt)))
     (Filename.quote stub_file)
+    (Filename.quote obj_file)
+    (Filename.quote !log_file))
+  &&
+  (* Now link the resulting .o with the LDFLAGS (lib) *)
+  (cmd
+    "%s -custom %s %s %s >> %s 2>&1"
+    !ocamlc
+    (Filename.quote obj_file)
     (Filename.quote !caml_file)
     (String.concat " " (List.map (sprintf "-cclib %s") lib))
-    (Filename.quote !log_file)
-  = 0
+    (Filename.quote !log_file))
 
 let safe_remove file_name =
-  try
-    Sys.remove file_name
-  with exn ->
-    ()
+  if !debug then
+    dprintf "DEBUG: Not removing %s\n" file_name
+  else
+    try
+      Sys.remove file_name
+    with exn ->
+      ()
 
 let test_code args stub_code =
   let stub_file, oc = Filename.open_temp_file "lwt_stub" ".c" in
   let cleanup () =
     safe_remove stub_file;
-    safe_remove (Filename.chop_extension (Filename.basename stub_file) ^ !ext_obj)
+    safe_remove (Filename.chop_extension stub_file ^ !ext_obj)
   in
   try
     output_string oc stub_code;
@@ -302,13 +388,13 @@ let test_feature ?(do_check = true) name macro test =
     if test () then begin
       if macro <> "" then begin
         fprintf config "#define %s\n" macro;
-        fprintf config_ml "#let %s = true\n" macro
+        fprintf config_ml "let _%s = true\n" macro
       end;
       printf " %s available\n%!" (String.make (34 - String.length name) '.')
     end else begin
       if macro <> "" then begin
         fprintf config "//#define %s\n" macro;
-        fprintf config_ml "#let %s = false\n" macro
+        fprintf config_ml "let _%s = false\n" macro
       end;
       printf " %s unavailable\n%!" (String.make (34 - String.length name) '.');
       not_available := name :: !not_available
@@ -317,7 +403,7 @@ let test_feature ?(do_check = true) name macro test =
     printf "not checking for %s\n%!" name;
     if macro <> "" then begin
       fprintf config "//#define %s\n" macro;
-      fprintf config_ml "#let %s = false\n" macro
+      fprintf config_ml "let _%s = false\n" macro
     end
   end
 
@@ -408,7 +494,9 @@ let () =
     "-use-pthread", arg_bool use_pthread, " whether to use pthread";
     "-use-unix", arg_bool use_unix, " whether to build lwt.unix";
     "-os-type", Arg.Set_string os_type, "<name> type of the target os";
+    "-android-target", arg_bool android_target, "<name> compiles for Android";
     "-ccomp-type", Arg.Set_string ccomp_type, "<ccomp-type> C compiler type";
+    "-libev_default", arg_bool libev_default, " whether to use the libev backend by default";
   ] in
   Arg.parse args ignore "check for external C libraries and available features\noptions are:";
 
@@ -464,6 +552,7 @@ let () =
 
   let test_pthread () =
     let opt, lib =
+      if !android_target then ([], []) else
       lib_flags "PTHREAD"
         (fun () ->
           match search_header "pthread.h" with
@@ -490,6 +579,28 @@ let () =
     test_code (opt, lib) glib_code
   in
 
+  let test_nanosecond_stat () =
+    printf "testing for nanosecond stat support:%!";
+    let conversions = [
+      ("(buf, field) buf->st_##field##tim.tv_nsec",      "*tim.tv_nsec");
+      ("(buf, field) buf->st_##field##timespec.tv_nsec", "*timespec.tv_nsec");
+      ("(buf, field) buf->st_##field##timensec",         "*timensec");
+    ] in
+    let fallback = "(buf, field) 0.0" in
+    let conversion = try
+      let (conversion, desc) = List.find (fun (conversion, _desc) ->
+        test_code ([], []) (struct_ns_code conversion)
+      ) conversions in
+      printf " %s %s\n%!" (String.make 11 '.') desc;
+      conversion
+    with Not_found -> begin
+      printf " %s unavailable\n%!" (String.make 11 '.');
+      fprintf config "#define NANOSEC%s\n" fallback;
+      fallback
+    end in
+    fprintf config "#define NANOSEC%s\n" conversion
+  in
+
   test_feature ~do_check:!use_libev "libev" "HAVE_LIBEV" test_libev;
   test_feature ~do_check:!use_pthread "pthread" "HAVE_PTHREAD" test_pthread;
   test_feature ~do_check:!use_glib "glib" "" test_glib;
@@ -500,12 +611,12 @@ let () =
     printf "
 The following recquired C libraries are missing: %s.
 Please install them and retry. If they are installed in a non-standard location
-or need special flags, set the environment variables <LIB>_CLFAGS and <LIB>_LIBS
+or need special flags, set the environment variables <LIB>_CFLAGS and <LIB>_LIBS
 accordingly and retry.
 
 For example, if libev is installed in /opt/local, you can type:
 
-export LIBEV_CLFAGS=-I/opt/local/include
+export LIBEV_CFLAGS=-I/opt/local/include
 export LIBEV_LIBS=-L/opt/local/lib
 
 To compile without libev support, use ./configure --disable-libev ...
@@ -527,15 +638,20 @@ Lwt can use pthread or the win32 API.
   let do_check = !os_type <> "Win32" in
   test_feature ~do_check "eventfd" "HAVE_EVENTFD" (fun () -> test_code ([], []) eventfd_code);
   test_feature ~do_check "fd passing" "HAVE_FD_PASSING" (fun () -> test_code ([], []) fd_passing_code);
-  test_feature ~do_check "sched_getcpu" "HAVE_GETCPU" (fun () -> test_code ([], []) getcpu_code);
-  test_feature ~do_check "affinity getting/setting" "HAVE_AFFINITY" (fun () -> test_code ([], []) affinity_code);
+  test_feature ~do_check:(do_check && not !android_target)
+    "sched_getcpu" "HAVE_GETCPU" (fun () -> test_code ([], []) getcpu_code);
+  test_feature ~do_check:(do_check && not !android_target)
+    "affinity getting/setting" "HAVE_AFFINITY" (fun () -> test_code ([], []) affinity_code);
   test_feature ~do_check "credentials getting (Linux)" "HAVE_GET_CREDENTIALS_LINUX" (fun () -> test_code ([], []) (get_credentials_code "ucred"));
   test_feature ~do_check "credentials getting (NetBSD)" "HAVE_GET_CREDENTIALS_NETBSD" (fun () -> test_code ([], []) (get_credentials_code "sockcred"));
   test_feature ~do_check "credentials getting (OpenBSD)" "HAVE_GET_CREDENTIALS_OPENBSD" (fun () -> test_code ([], []) (get_credentials_code "sockpeercred"));
   test_feature ~do_check "credentials getting (FreeBSD)" "HAVE_GET_CREDENTIALS_FREEBSD" (fun () -> test_code ([], []) (get_credentials_code "cmsgcred"));
   test_feature ~do_check "credentials getting (getpeereid)" "HAVE_GETPEEREID" (fun () -> test_code ([], []) get_peereid_code);
   test_feature ~do_check "fdatasync" "HAVE_FDATASYNC" (fun () -> test_code ([], []) fdatasync_code);
-  test_feature ~do_check "netdb_reentrant" "HAVE_NETDB_REENTRANT" (fun () -> test_code ([], []) netdb_reentrant_code);
+  test_feature ~do_check:(do_check && not !android_target)
+    "netdb_reentrant" "HAVE_NETDB_REENTRANT" (fun () -> test_code ([], []) netdb_reentrant_code);
+  test_feature ~do_check "reentrant gethost*" "HAVE_REENTRANT_HOSTENT" (fun () -> test_code ([], []) hostent_reentrant_code);
+  test_nanosecond_stat ();
 
   let get_cred_vars = [
     "HAVE_GET_CREDENTIALS_LINUX";
@@ -553,15 +669,23 @@ Lwt can use pthread or the win32 API.
     (String.concat " || " (List.map (Printf.sprintf "defined(%s)") get_cred_vars));
 
   Printf.fprintf config_ml
-    "#let HAVE_GET_CREDENTIALS = %s\n"
-    (String.concat " || " get_cred_vars);
+    "let _HAVE_GET_CREDENTIALS = %s\n"
+    (String.concat " || " (List.map (fun s -> "_" ^ s) get_cred_vars));
 
   if !os_type = "Win32" then begin
     output_string config "#define LWT_ON_WINDOWS\n";
-    output_string config_ml "#let windows=true\n"
   end else begin
     output_string config "//#define LWT_ON_WINDOWS\n";
-    output_string config_ml "#let windows=false\n"
+  end;
+  if !android_target then begin
+    output_string config_ml "let android = true\n"
+  end else begin
+    output_string config_ml "let android = false\n"
+  end;
+  if !libev_default then begin
+    output_string config_ml "let libev_default = true\n"
+  end else begin
+    output_string config_ml "let libev_default = false\n"
   end;
 
   fprintf config "#endif\n";
