@@ -1,9 +1,9 @@
-(* Lightweight thread library for Objective Caml
+(* Lightweight thread library for OCaml
  * http://www.ocsigen.org/lwt
  * Module Lwt
- * Copyright (C) 2005-2008 Jrme Vouillon
- * Laboratoire PPS - CNRS Universit Paris Diderot
- *               2009-2012 Jrmie Dimino
+ * Copyright (C) 2005-2008 Jérôme Vouillon
+ * Laboratoire PPS - CNRS Université Paris Diderot
+ *               2009-2012 Jérémie Dimino
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as
@@ -518,6 +518,7 @@ let return v =
 let state_return_unit = Return ()
 let return_unit = thread { state = state_return_unit }
 let return_none = return None
+let return_some x = return (Some x)
 let return_nil = return []
 let return_true = return true
 let return_false = return false
@@ -528,6 +529,12 @@ let of_result result =
 let fail e =
   thread { state = Fail e }
 
+let fail_with msg =
+  thread { state = Fail (Failure msg) }
+
+let fail_invalid_arg msg =
+  thread { state = Fail (Invalid_argument msg) }
+
 let temp t =
   thread {
     state = Sleep { cancel = Cancel_link (pack_thread (thread t));
@@ -1182,7 +1189,9 @@ let pause () =
   waiter
 
 let wakeup_paused () =
-  if not (Lwt_sequence.is_empty paused) then begin
+  if Lwt_sequence.is_empty paused then
+    paused_count := 0
+  else begin
     let tmp = Lwt_sequence.create () in
     Lwt_sequence.transfer_r paused tmp;
     paused_count := 0;
@@ -1288,3 +1297,13 @@ let state t = match (repr t).state with
   | Repr _ -> assert false
 
 include State
+
+
+module Infix = struct
+  let (>>=) = (>>=)
+  let (=<<) = (=<<)
+  let (>|=) = (>|=)
+  let (=|<) = (=|<)
+  let (<&>) = (<&>)
+  let (<?>) = (<?>)
+end
