@@ -1,5 +1,5 @@
 (******************************************************************************)
-(* Lightweight thread library for Objective Caml
+(* Lightweight thread library for OCaml
  * http://www.ocsigen.org/lwt
  * Module Lwt_condition
  ******************************************************************************
@@ -29,7 +29,7 @@
  * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  ******************************************************************************)
 
-let (>>=) = Lwt.(>>=)
+open Lwt.Infix
 
 type 'a t = 'a Lwt.u Lwt_sequence.t
 
