@@ -1,4 +1,4 @@
-(* Lightweight thread library for Objective Caml
+(* Lightweight thread library for OCaml
  * http://www.ocsigen.org/lwt
  * Module Lwt_stream
  * Copyright (C) 2009 Jérémie Dimino
@@ -20,8 +20,7 @@
  * 02111-1307, USA.
  *)
 
-let (>>=) = Lwt.(>>=)
-let (>|=) = Lwt.(>|=)
+open Lwt.Infix
 
 exception Closed
 exception Full
