@@ -1,4 +1,4 @@
-(* Lightweight thread library for Objective Caml
+(* Lightweight thread library for OCaml
  * http://www.ocsigen.org/lwt
  * Module Lwt_stream
  * Copyright (C) 2009 Jérémie Dimino
@@ -32,7 +32,7 @@ type 'a t
     - [_p] when the function returns a thread and calls are parallelised
 *)
 
-(** {6 Construction} *)
+(** {2 Construction} *)
 
 val from : (unit -> 'a option Lwt.t) -> 'a t
   (** [from f] creates an stream from the given input function. [f] is
@@ -42,7 +42,7 @@ val from : (unit -> 'a option Lwt.t) -> 'a t
 val from_direct : (unit -> 'a option) -> 'a t
   (** [from_direct f] does the same as {!from} but with a function
       that does not return a thread. It is better than wrapping [f]
-      into a function which return a thread. *)
+      into a function which returns a thread. *)
 
 exception Closed
   (** Exception raised by the push function of a push-stream when
@@ -54,8 +54,8 @@ val create : unit -> 'a t * ('a option -> unit)
 
 val create_with_reference : unit -> 'a t * ('a option -> unit) * ('b -> unit)
   (** [create_with_reference ()] returns a new stream and a push
-      function. The last function allows to set a reference to an
-      external source. This prevent the external source from being
+      function. The last function allows a reference to be set to an
+      external source. This prevents the external source from being
       garbage collected.
 
       For example, to convert a reactive event to a stream:
@@ -146,7 +146,7 @@ val clone : 'a t -> 'a t
       It raises [Invalid_argument] if [st] is a bounded
       push-stream. *)
 
-(** {6 Destruction} *)
+(** {2 Destruction} *)
 
 val to_list : 'a t -> 'a list Lwt.t
   (** Returns the list of elements of the given stream *)
@@ -155,7 +155,7 @@ val to_string : char t -> string Lwt.t
   (** Returns the word composed of all characters of the given
       stream *)
 
-(** {6 Data retreival} *)
+(** {2 Data retreival} *)
 
 exception Empty
   (** Exception raised when trying to retreive data from an empty
@@ -229,7 +229,7 @@ val on_terminate : 'a t -> (unit -> unit) -> unit
       is reached. Note that the stream may still contains elements if
       {!peek} or similar was used. *)
 
-(** {6 Stream transversal} *)
+(** {2 Stream transversal} *)
 
 (** Note: all the following functions are destructive.
 
@@ -312,7 +312,7 @@ val map_exn : 'a t -> 'a result t
       Note that for push-streams (as returned by {!create}) all
       elements of the mapped streams are values. *)
 
-(** {6 Parsing} *)
+(** {2 Parsing} *)
 
 val parse : 'a t -> ('a t -> 'b Lwt.t) -> 'b Lwt.t
   (** [parse st f] parses [st] with [f]. If [f] raise an exception,
@@ -321,7 +321,7 @@ val parse : 'a t -> ('a t -> 'b Lwt.t) -> 'b Lwt.t
       It raises [Invalid_argument] if [st] is a bounded
       push-stream. *)
 
-(** {6 Misc} *)
+(** {2 Misc} *)
 
 val hexdump : char t -> string t
   (** [hexdump byte_stream] returns a stream which is the same as the
