@@ -1,9 +1,9 @@
 (* Ocsigen
  * http://www.ocsigen.org
  * Module lwt_preemptive.ml
- * Copyright (C) 2005 Nataliya Guts, Vincent Balat, Jrme Vouillon
- * Laboratoire PPS - CNRS Universit Paris Diderot
- *               2009 Jrmie Dimino
+ * Copyright (C) 2005 Nataliya Guts, Vincent Balat, Jérôme Vouillon
+ * Laboratoire PPS - CNRS Université Paris Diderot
+ *               2009 Jérémie Dimino
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -23,7 +23,7 @@
 
 (** This module allows to mix preemptive threads with [Lwt]
     cooperative threads. It maintains an extensible pool of preemptive
-    threads to with you can detach computations. *)
+    threads to which you can detach computations. *)
 
 val detach : ('a -> 'b) -> 'a -> 'b Lwt.t
   (** detaches a computation to a preemptive thread. *)
