@@ -1,4 +1,4 @@
-(* Lightweight thread library for Objective Caml
+(* Lightweight thread library for OCaml
  * http://www.ocsigen.org/lwt
  * Module Lwt_unix
  * Copyright (C) 2010 Jérémie Dimino
@@ -21,10 +21,9 @@
  * 02111-1307, USA.
  *)
 
-#include "src/unix/lwt_config.ml"
-
 open Bigarray
-open Lwt
+
+open Lwt.Infix
 
 type t = (char, int8_unsigned_elt, c_layout) Array1.t
 
@@ -49,25 +48,25 @@ let fill bytes ofs len ch =
    | Blitting                                                        |
    +-----------------------------------------------------------------+ *)
 
-external unsafe_blit_string_bytes : string -> int -> t -> int -> int -> unit = "lwt_unix_blit_string_bytes" "noalloc"
-external unsafe_blit_bytes_string : t -> int -> string -> int -> int -> unit = "lwt_unix_blit_bytes_string" "noalloc"
-external unsafe_blit : t -> int -> t -> int -> int -> unit = "lwt_unix_blit_bytes_bytes" "noalloc"
+external unsafe_blit_from_bytes : Bytes.t -> int -> t -> int -> int -> unit = "lwt_unix_blit_from_bytes" "noalloc"
+external unsafe_blit_to_bytes : t -> int -> Bytes.t -> int -> int -> unit = "lwt_unix_blit_to_bytes" "noalloc"
+external unsafe_blit : t -> int -> t -> int -> int -> unit = "lwt_unix_blit" "noalloc"
 
-let blit_string_bytes src_buf src_ofs dst_buf dst_ofs len =
+let blit_from_bytes src_buf src_ofs dst_buf dst_ofs len =
   if (len < 0
-      || src_ofs < 0 || src_ofs > String.length src_buf - len
+      || src_ofs < 0 || src_ofs > Bytes.length src_buf - len
       || dst_ofs < 0 || dst_ofs > length dst_buf - len) then
     invalid_arg "String.blit"
   else
-    unsafe_blit_string_bytes src_buf src_ofs dst_buf dst_ofs len
+    unsafe_blit_from_bytes src_buf src_ofs dst_buf dst_ofs len
 
-let blit_bytes_string src_buf src_ofs dst_buf dst_ofs len =
+let blit_to_bytes src_buf src_ofs dst_buf dst_ofs len =
   if (len < 0
       || src_ofs < 0 || src_ofs > length src_buf - len
-      || dst_ofs < 0 || dst_ofs > String.length dst_buf - len) then
+      || dst_ofs < 0 || dst_ofs > Bytes.length dst_buf - len) then
     invalid_arg "String.blit"
   else
-    unsafe_blit_bytes_string src_buf src_ofs dst_buf dst_ofs len
+    unsafe_blit_to_bytes src_buf src_ofs dst_buf dst_ofs len
 
 let blit src_buf src_ofs dst_buf dst_ofs len =
   if (len < 0
@@ -77,18 +76,22 @@ let blit src_buf src_ofs dst_buf dst_ofs len =
   else
     unsafe_blit src_buf src_ofs dst_buf dst_ofs len
 
-let of_string str =
-  let len = String.length str in
+let of_bytes buf =
+  let len = Bytes.length buf in
   let bytes = create len in
-  unsafe_blit_string_bytes str 0 bytes 0 len;
+  unsafe_blit_from_bytes buf 0 bytes 0 len;
   bytes
 
-let to_string bytes =
+let of_string str = of_bytes (Bytes.unsafe_of_string str)
+
+let to_bytes bytes =
   let len = length bytes in
-  let str = String.create len in
-  unsafe_blit_bytes_string bytes 0 str 0 len;
+  let str = Bytes.create len in
+  unsafe_blit_to_bytes bytes 0 str 0 len;
   str
 
+let to_string bytes = Bytes.unsafe_to_string (to_bytes bytes)
+
 let proxy = Array1.sub
 
 let extract buf ofs len =
@@ -121,7 +124,7 @@ let read fd buf pos len =
   else
     blocking fd >>= function
       | true ->
-          lwt () = wait_read fd in
+          wait_read fd >>= fun () ->
           run_job (read_job (unix_file_descr fd) buf pos len)
       | false ->
           wrap_syscall Read fd (fun () -> stub_read (unix_file_descr fd) buf pos len)
@@ -135,18 +138,11 @@ let write fd buf pos len =
   else
     blocking fd >>= function
       | true ->
-          lwt () = wait_write fd in
+          wait_write fd >>= fun () ->
           run_job (write_job (unix_file_descr fd) buf pos len)
       | false ->
           wrap_syscall Write fd (fun () -> stub_write (unix_file_descr fd) buf pos len)
 
-#if windows
-
-let recv fd buf pos len flags =
-  raise (Lwt_sys.Not_available "Lwt_bytes.recv")
-
-#else
-
 external stub_recv : Unix.file_descr -> t -> int -> int -> Unix.msg_flag list -> int = "lwt_unix_bytes_recv"
 
 let recv fd buf pos len flags =
@@ -155,15 +151,6 @@ let recv fd buf pos len flags =
   else
     wrap_syscall Read fd (fun () -> stub_recv (unix_file_descr fd) buf pos len flags)
 
-#endif
-
-#if windows
-
-let send fd buf pos len flags =
-  raise (Lwt_sys.Not_available "Lwt_bytes.send")
-
-#else
-
 external stub_send : Unix.file_descr -> t -> int -> int -> Unix.msg_flag list -> int = "lwt_unix_bytes_send"
 
 let send fd buf pos len flags =
@@ -172,8 +159,6 @@ let send fd buf pos len flags =
   else
     wrap_syscall Write fd (fun () -> stub_send (unix_file_descr fd) buf pos len flags)
 
-#endif
-
 type io_vector = {
   iov_buffer : t;
   iov_offset : int;
@@ -195,13 +180,6 @@ let check_io_vectors func_name iovs =
            invalid_arg func_name)
     iovs
 
-#if windows
-
-let recv_msg ~socket ~io_vectors =
-  raise (Lwt_sys.Not_available "recv_msg")
-
-#else
-
 external stub_recv_msg : Unix.file_descr -> int -> io_vector list -> int * Unix.file_descr list = "lwt_unix_bytes_recv_msg"
 
 let recv_msg ~socket ~io_vectors =
@@ -211,15 +189,6 @@ let recv_msg ~socket ~io_vectors =
     (fun () ->
        stub_recv_msg (unix_file_descr socket) n_iovs io_vectors)
 
-#endif
-
-#if windows
-
-let send_msg ~socket ~io_vectors ~fds =
-  raise (Lwt_sys.Not_available "send_msg")
-
-#else
-
 external stub_send_msg : Unix.file_descr -> int -> io_vector list -> int -> Unix.file_descr list -> int = "lwt_unix_bytes_send_msg"
 
 let send_msg ~socket ~io_vectors ~fds =
@@ -229,15 +198,6 @@ let send_msg ~socket ~io_vectors ~fds =
     (fun () ->
        stub_send_msg (unix_file_descr socket) n_iovs io_vectors n_fds fds)
 
-#endif
-
-#if windows
-
-let recvfrom fd buf pos len flags =
-  raise (Lwt_sys.Not_available "Lwt_bytes.recvfrom")
-
-#else
-
 external stub_recvfrom : Unix.file_descr -> t -> int -> int -> Unix.msg_flag list -> int * Unix.sockaddr = "lwt_unix_bytes_recvfrom"
 
 let recvfrom fd buf pos len flags =
@@ -246,15 +206,6 @@ let recvfrom fd buf pos len flags =
   else
     wrap_syscall Read fd (fun () -> stub_recvfrom (unix_file_descr fd) buf pos len flags)
 
-#endif
-
-#if windows
-
-let sendto fd buf pos len flags addr =
-  raise (Lwt_sys.Not_available "Lwt_bytes.sendto")
-
-#else
-
 external stub_sendto : Unix.file_descr -> t -> int -> int -> Unix.msg_flag list -> Unix.sockaddr -> int = "lwt_unix_bytes_sendto_byte" "lwt_unix_bytes_sendto"
 
 let sendto fd buf pos len flags addr =
@@ -263,8 +214,6 @@ let sendto fd buf pos len flags addr =
   else
     wrap_syscall Write fd (fun () -> stub_sendto (unix_file_descr fd) buf pos len flags addr)
 
-#endif
-
 (* +-----------------------------------------------------------------+
    | Memory mapped files                                             |
    +-----------------------------------------------------------------+ *)
@@ -281,13 +230,6 @@ type advice =
   | MADV_WILLNEED
   | MADV_DONTNEED
 
-#if windows
-
-let madvise buf pos len advice =
-  raise (Lwt_sys.Not_available "madvise")
-
-#else
-
 external stub_madvise : t -> int -> int -> advice -> unit = "lwt_unix_madvise"
 
 let madvise buf pos len advice =
@@ -296,22 +238,10 @@ let madvise buf pos len advice =
   else
     stub_madvise buf pos len advice
 
-#endif
-
 external get_page_size : unit -> int = "lwt_unix_get_page_size"
 
 let page_size = get_page_size ()
 
-#if windows
-
-let mincore buffer offset states =
-  raise (Lwt_sys.Not_available "mincore")
-
-let wait_mincore buffer offset =
-  raise (Lwt_sys.Not_available "mincore")
-
-#else
-
 external stub_mincore : t -> int -> int -> bool array -> unit = "lwt_unix_mincore"
 
 let mincore buffer offset states =
@@ -331,9 +261,7 @@ let wait_mincore buffer offset =
     let state = [|false|] in
     mincore buffer (offset - (offset mod page_size)) state;
     if state.(0) then
-      return ()
+      Lwt.return_unit
     else
       run_job (wait_mincore_job buffer offset)
   end
-
-#endif
