@@ -1,8 +1,8 @@
-(* Lightweight thread library for Objective Caml
+(* Lightweight thread library for OCaml
  * http://www.ocsigen.org/lwt
  * Interface Lwt_chan
- * Copyright (C) 2005-2008 Jrme Vouillon
- * Laboratoire PPS - CNRS Universit Paris Diderot
+ * Copyright (C) 2005-2008 Jérôme Vouillon
+ * Laboratoire PPS - CNRS Université Paris Diderot
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as
@@ -25,12 +25,13 @@
 
 (** Note: the {!Lwt_io} module deprecates this module. *)
 
-(** {2 Cooperative input channels} *)
+(** {3 Cooperative input channels} *)
 type in_channel = Lwt_io.input_channel
 
 val in_channel_of_descr : Lwt_unix.file_descr -> in_channel
 
-val make_in_channel : ?close:(unit -> unit Lwt.t) -> (string -> int -> int -> int Lwt.t) -> in_channel
+val make_in_channel : ?close:(unit -> unit Lwt.t) ->
+  (Bytes.t -> int -> int -> int Lwt.t) -> in_channel
 (** [make_in_channel read] creates an input channel from the [read]
     function. [read s ofs len] should (cooperatively) read [len] bytes from
     the source, and put them in [s], at offset [ofs], and return the number
@@ -39,8 +40,8 @@ val make_in_channel : ?close:(unit -> unit Lwt.t) -> (string -> int -> int -> in
 
 val input_line : in_channel -> string Lwt.t
 val input_value : in_channel -> 'a Lwt.t
-val input : in_channel -> string -> int -> int -> int Lwt.t
-val really_input : in_channel -> string -> int -> int -> unit Lwt.t
+val input : in_channel -> Bytes.t -> int -> int -> int Lwt.t
+val really_input : in_channel -> Bytes.t -> int -> int -> unit Lwt.t
 val input_char : in_channel -> char Lwt.t
 val input_binary_int : in_channel -> int Lwt.t
 
@@ -48,20 +49,21 @@ val open_in_gen : Unix.open_flag list -> int -> string -> in_channel Lwt.t
 val open_in : string -> in_channel Lwt.t
 val close_in : in_channel -> unit Lwt.t
 
-(** {2 Cooperative output channels} *)
+(** {3 Cooperative output channels} *)
 
 type out_channel = Lwt_io.output_channel
 
 val out_channel_of_descr : Lwt_unix.file_descr -> out_channel
 
-val make_out_channel : ?close:(unit -> unit Lwt.t) -> (string -> int -> int -> int Lwt.t) -> out_channel
+val make_out_channel : ?close:(unit -> unit Lwt.t) ->
+  (Bytes.t -> int -> int -> int Lwt.t) -> out_channel
 (** [make_out_channel write] creates an output channel from the [write]
     function. [write s ofs len] should (cooperatively) write [len] bytes from
     [s], starting at offset [ofs], to the backend, and return the number of
     bytes effectively written. If provided, [close] will be called by
     [close_out]. By default, [close_out] does not do anything. *)
 
-val output : out_channel -> string -> int -> int -> unit Lwt.t
+val output : out_channel -> Bytes.t -> int -> int -> unit Lwt.t
 val flush : out_channel -> unit Lwt.t
 val output_string : out_channel -> string -> unit Lwt.t
 val output_value : out_channel -> 'a -> unit Lwt.t
