@@ -1,4 +1,4 @@
-(* Lightweight thread library for Objective Caml
+(* Lightweight thread library for OCaml
  * http://www.ocsigen.org/lwt
  * Module Lwt_io
  * Copyright (C) 2009 Jérémie Dimino
@@ -20,11 +20,11 @@
  * 02111-1307, USA.
  *)
 
-open Lwt
+open Lwt.Infix
 
 let rec copy ic logger =
-  lwt line = Lwt_io.read_line ic in
-  lwt () = Lwt_log.log ?logger ~level:Lwt_log.Notice line in
+  Lwt_io.read_line ic >>= fun line ->
+  Lwt_log.log ?logger ~level:Lwt_log.Notice line >>= fun () ->
   copy ic logger
 
 let redirect fd logger =
@@ -32,8 +32,8 @@ let redirect fd logger =
   Unix.set_close_on_exec fd_r;
   Unix.dup2 fd_w fd;
   Unix.close fd_w;
-  let ic = Lwt_io.of_unix_fd ~mode:Lwt_io.input fd in
-  ignore (copy ic logger)
+  let ic = Lwt_io.of_unix_fd ~mode:Lwt_io.input fd_r in
+  Lwt.ignore_result (copy ic logger)
 
 let redirect_output dev_null fd mode = match mode with
   | `Dev_null ->
@@ -81,7 +81,7 @@ let daemonize ?(syslog=true) ?(stdin=`Dev_null) ?(stdout=`Log_default) ?(stderr=
       | `Keep ->
           ()
       | `Set n ->
-          ignore (Unix.umask 0o022);
+          ignore (Unix.umask n);
     end;
 
     ignore (Unix.setsid ())
