@@ -1,4 +1,4 @@
-(* Lightweight thread library for Objective Caml
+(* Lightweight thread library for OCaml
  * http://www.ocsigen.org/lwt
  * Module Lwt_engine
  * Copyright (C) 2011 Jérémie Dimino
@@ -20,8 +20,6 @@
  * 02111-1307, USA.
  *)
 
-#include "src/unix/lwt_config.ml"
-
 (* +-----------------------------------------------------------------+
    | Events                                                          |
    +-----------------------------------------------------------------+ *)
@@ -126,8 +124,6 @@ end
    | The libev engine                                                |
    +-----------------------------------------------------------------+ *)
 
-#if HAVE_LIBEV
-
 type ev_loop
 type ev_io
 type ev_timer
@@ -170,24 +166,6 @@ class libev = object
     lazy(ev_timer_stop loop ev)
 end
 
-#else
-
-type ev_loop
-
-class libev = object(self)
-  inherit abstract
-
-  val loop : ev_loop = raise (Lwt_sys.Not_available "libev")
-  method loop : ev_loop = assert false
-  method iter = assert false
-  method private cleanup = assert false
-  method private register_readable = assert false
-  method private register_writable = assert false
-  method private register_timer = assert false
-end
-
-#endif
-
 (* +-----------------------------------------------------------------+
    | Select/poll based engines                                       |
    +-----------------------------------------------------------------+ *)
@@ -217,8 +195,13 @@ let rec restart_actions sleep_queue now =
     | Some{ stopped = true } ->
         restart_actions (Sleep_queue.remove_min sleep_queue) now
     | Some{ time = time; action = action } when time <= now ->
+        (* We have to remove the sleeper to the queue before performing
+           the action. The action can change the sleeper's time, and this
+           might break the priority queue invariant if the sleeper is
+           still in the queue. *)
+        let q = Sleep_queue.remove_min sleep_queue in
         action ();
-        restart_actions (Sleep_queue.remove_min sleep_queue) now
+        restart_actions q now
     | _ ->
         sleep_queue
 
@@ -252,8 +235,8 @@ class virtual select_or_poll_based = object(self)
   val mutable new_sleeps = []
     (* Sleepers added since the last iteration of the main loop:
 
-       They are not added immediatly to the main sleep queue in order
-       to prevent them from being wakeup immediatly.  *)
+       They are not added immediately to the main sleep queue in order
+       to prevent them from being wakeup immediately.  *)
 
   val mutable wait_readable = Fd_map.empty
     (* Sequences of actions waiting for file descriptors to become
@@ -393,15 +376,11 @@ end
    | The current engine                                              |
    +-----------------------------------------------------------------+ *)
 
-#if HAVE_LIBEV && not windows
-
-let current = ref (new libev :> t)
-
-#else
-
-let current = ref (new select :> t)
-
-#endif
+let current =
+  if Lwt_config._HAVE_LIBEV && Lwt_config.libev_default then
+    ref (new libev :> t)
+  else
+    ref (new select :> t)
 
 let get () =
   !current
