@@ -1,4 +1,4 @@
-(* Lightweight thread library for Objective Caml
+(* Lightweight thread library for OCaml
  * http://www.ocsigen.org/lwt
  * Interface Lwt_engine
  * Copyright (C) 2011 Jérémie Dimino
@@ -22,7 +22,7 @@
 
 (** Lwt unix main loop engine *)
 
-(** {6 Events} *)
+(** {2 Events} *)
 
 type event
   (** Type of events. An event represent a callback registered to be
@@ -34,13 +34,13 @@ val stop_event : event -> unit
 val fake_event : event
   (** Event which does nothing when stopped. *)
 
-(** {6 Event loop functions} *)
+(** {2 Event loop functions} *)
 
 val iter : bool -> unit
   (** [iter block] performs one iteration of the main loop. If [block]
-      is [true] the function must blocks until one event become
+      is [true] the function must block until one event becomes
       available, otherwise it should just check for available events
-      and return immediatly. *)
+      and return immediately. *)
 
 val on_readable : Unix.file_descr -> (event -> unit) -> event
   (** [on_readable fd f] calls [f] each time [fd] becomes readable. *)
@@ -67,10 +67,10 @@ val timer_count : unit -> int
 val fake_io : Unix.file_descr -> unit
   (** Simulates activity on the given file descriptor. *)
 
-(** {6 Engines} *)
+(** {2 Engines} *)
 
-(** An engine represent a set of functions used to register different
-    kind of callbacks for different kind of events. *)
+(** An engine represents a set of functions used to register different
+    kinds of callbacks for different kinds of events. *)
 
 (** Abstract class for engines. *)
 class virtual abstract : object
@@ -82,10 +82,10 @@ class virtual abstract : object
     (** [transfer engine] moves all events from the current engine to
         [engine]. Note that timers are reset in the destination
         engine, i.e. if a timer with a delay of 2 seconds was
-        registered 1 second ago it will occurs in 2 seconds in the
+        registered 1 second ago it will occur in 2 seconds in the
         destination engine. *)
 
-  (** {6 Event loop methods} *)
+  (** {2 Event loop methods} *)
 
   method virtual iter : bool -> unit
   method on_readable : Unix.file_descr -> (event -> unit) -> event
@@ -96,18 +96,18 @@ class virtual abstract : object
   method writable_count : int
   method timer_count : int
 
-  (** {6 Backend methods} *)
+  (** {2 Backend methods} *)
 
   (** Notes:
 
       - the callback passed to register methods is of type [unit ->
       unit] and not [event -> unit]
-      - register methods returns a lazy value which unregister the
+      - register methods return a lazy value which unregisters the
       event when forced
   *)
 
   method virtual private cleanup : unit
-    (** Cleanup resources associated to the engine. *)
+    (** Cleanup resources associated with the engine. *)
 
   method virtual private register_readable : Unix.file_descr -> (unit -> unit) -> unit Lazy.t
   method virtual private register_writable : Unix.file_descr -> (unit -> unit) -> unit Lazy.t
@@ -125,7 +125,7 @@ class type t = object
   method private register_timer : float -> bool -> (unit -> unit) -> unit Lazy.t
 end
 
-(** {6 Predefined engines} *)
+(** {2 Predefined engines} *)
 
 type ev_loop
   (** Type of libev loops. *)
@@ -178,7 +178,7 @@ class virtual poll_based : object
         and writable status. *)
 end
 
-(** {6 The current engine} *)
+(** {2 The current engine} *)
 
 val get : unit -> t
   (** [get ()] returns the engine currently in use. *)
