@@ -1,4 +1,4 @@
-(* Lightweight thread library for Objective Caml
+(* Lightweight thread library for OCaml
  * http://www.ocsigen.org/lwt
  * Interface Lwt_io
  * Copyright (C) 2009 Jérémie Dimino
@@ -22,26 +22,25 @@
 
 (** Buffered byte channels *)
 
-(** A {b channel} is a high-level object for performing IOs. It allow
-    to read/write things from/to the outside worlds in an efficient
-    way, by minimising the number of system calls.
+(** A {b channel} is a high-level object for performing input/output
+    (IO). It allows to read/write from/to the outside world in an
+    efficient way, by minimising the number of system calls.
 
-    An {b output channel} is a channel that can be used to send data
-    and an {b input channel} is a channel that can used to receive
-    data.
+    An {b output channel} is used to send data and an {b input
+    channel} is used to receive data.
 
     If you are familiar with buffered channels you may be familiar too
-    with the {b flush} operation. Note that byte channles of this
-    modules are automatically flushed when there is nothing else to do
-    (i.e. before the program goes into idle), so this means that you
-    no longer have to write:
+    with the {b flush} operation. Note that byte channels of this
+    module are automatically flushed when there is nothing else to do
+    (i.e. before the program becomes idle), so this means that you no
+    longer have to write:
 
     {[
       eprintf "log message\n";
       flush stderr;
     ]}
 
-    to have you messages displayed.
+    to have your messages displayed.
 
     Note about errors: input functions of this module raise
     [End_of_file] when the end-of-file is reached (i.e. when the read
@@ -53,7 +52,7 @@ exception Channel_closed of string
   (** Exception raised when a channel is closed. The parameter is a
       description of the channel. *)
 
-(** {6 Types} *)
+(** {2 Types} *)
 
 type 'mode channel
   (** Type of buffered byte channels *)
@@ -65,16 +64,9 @@ type output
   (** Output mode *)
 
 (** Channel mode *)
-#if ocaml_version >= (3, 13)
 type 'a mode =
   | Input : input mode
   | Output : output mode
-#else
-type 'a mode =
-    private
-  | Input
-  | Output
-#endif
 
 val input : input mode
   (** [input] input mode representation *)
@@ -91,7 +83,7 @@ type output_channel = output channel
 val mode : 'a channel -> 'a mode
   (** [mode ch] returns the mode of a channel *)
 
-(** {6 Well-known instances} *)
+(** {2 Well-known instances} *)
 
 val stdin : input_channel
   (** The standard input, it reads data from {!Lwt_unix.stdin} *)
@@ -109,23 +101,35 @@ val zero : input_channel
 val null : output_channel
   (** Output which drops everything *)
 
-(** {6 Channels creation/manipulation} *)
+(** {2 Channels creation/manipulation} *)
 
-val pipe : ?buffer_size : int -> unit -> input_channel * output_channel
-  (** [pipe ?buffer_size ()] creates a pipe using {!Lwt_unix.pipe} and
-      makes two channels from the two returned file descriptors *)
+val pipe : ?in_buffer : Lwt_bytes.t -> ?out_buffer : Lwt_bytes.t -> unit ->
+  input_channel * output_channel
+  (** [pipe ?in_buffer ?out_buffer ()] creates a pipe using
+      {!Lwt_unix.pipe} and makes two channels from the two returned file
+      descriptors *)
 
 val make :
-  ?buffer_size : int ->
+  ?buffer : Lwt_bytes.t ->
   ?close : (unit -> unit Lwt.t) ->
   ?seek : (int64 -> Unix.seek_command -> int64 Lwt.t) ->
   mode : 'mode mode ->
   (Lwt_bytes.t -> int -> int -> int Lwt.t) -> 'mode channel
-  (** [make ?buffer_size ?close ~mode perform_io] is the
+  (** [make ?buffer ?close ~mode perform_io] is the
       main function for creating new channels.
 
-      @param buffer_size size of the internal buffer. It must be
-      between 16 and [Sys.max_string_length]
+      @param buffer user-supplied buffer. When this argument is
+      present, its value will be used as the buffer for the created
+      channel. The size of buffer must conform to the limitations
+      described in {!set_default_buffer_size}.  When this argument is
+      not present, a new internal buffer of default size will be
+      allocated for this channel.
+
+      Warning: do not use the same buffer for simultaneous work with more
+      than one channel.
+
+      There are other functions in this module that take a [buffer]
+      argument, sharing the same semantics.
 
       @param close close function of the channel. It defaults to
       [Lwt.return]
@@ -142,32 +146,34 @@ val of_bytes : mode : 'mode mode -> Lwt_bytes.t -> 'mode channel
   (** Create a channel from a byte array. Reading/writing is done
       directly on the provided array. *)
 
-val of_fd : ?buffer_size : int -> ?close : (unit -> unit Lwt.t) -> mode : 'mode mode -> Lwt_unix.file_descr -> 'mode channel
-  (** [of_fd ?buffer_size ?close ~mode fd] creates a channel from a
+val of_fd : ?buffer : Lwt_bytes.t -> ?close : (unit -> unit Lwt.t) ->
+  mode : 'mode mode -> Lwt_unix.file_descr -> 'mode channel
+  (** [of_fd ?buffer ?close ~mode fd] creates a channel from a
       file descriptor.
 
       @param close defaults to closing the file descriptor. *)
 
-val of_unix_fd : ?buffer_size : int -> ?close : (unit -> unit Lwt.t) -> mode : 'mode mode -> Unix.file_descr -> 'mode channel
-  (** [of_unix_fd ?buffer_size ?close ~mode fd] is a short-hand for:
+val of_unix_fd : ?buffer : Lwt_bytes.t -> ?close : (unit -> unit Lwt.t) ->
+  mode : 'mode mode -> Unix.file_descr -> 'mode channel
+  (** [of_unix_fd ?buffer ?close ~mode fd] is a short-hand for:
 
-      [of_fd ?buffer_size ?close (Lwt_unix.of_unix_file_descr fd)] *)
+      [of_fd ?buffer ?close (Lwt_unix.of_unix_file_descr fd)] *)
 
 val close : 'a channel -> unit Lwt.t
   (** [close ch] closes the given channel. If [ch] is an output
-      channel, it performs all pending actions, flush it and close
-      it. If [ch] is an input channel, it just close it immediatly.
+      channel, it performs all pending actions, flushes it and closes
+      it. If [ch] is an input channel, it just closes it immediately.
 
       [close] returns the result of the close function of the
       channel. Multiple calls to [close] will return exactly the same
       value.
 
-      Note: you cannot use [close] on channel obtained with an
+      Note: you cannot use [close] on channels obtained with
       {!atomic}. *)
 
 val abort : 'a channel -> unit Lwt.t
   (** [abort ch] abort current operations and close the channel
-      immediatly. *)
+      immediately. *)
 
 val atomic : ('a channel -> 'b Lwt.t) -> ('a channel -> 'b Lwt.t)
   (** [atomic f] transforms a sequence of io operations into one
@@ -200,19 +206,19 @@ val is_busy : 'a channel -> bool
       busy. A channel is busy when there is at least one job using it
       that has not yet terminated. *)
 
-(** {6 Random access} *)
+(** {2 Random access} *)
 
 val position : 'a channel -> int64
   (** [position ch] Returns the current position in the channel. *)
 
 val set_position : 'a channel -> int64 -> unit Lwt.t
   (** [set_position ch pos] Sets the position in the output channel. This
-      does not work if the channel do not support random access. *)
+      does not work if the channel does not support random access. *)
 
 val length : 'a channel -> int64 Lwt.t
   (** Returns the length of the channel in bytes *)
 
-(** {6 Reading} *)
+(** {2 Reading} *)
 
 (** Note: except for functions dealing with streams ({!read_chars} and
     {!read_lines}) all functions are {b atomic}. *)
@@ -223,7 +229,7 @@ val read_char : input_channel -> char Lwt.t
       @raise End_of_file if the end of the file is reached *)
 
 val read_char_opt : input_channel -> char option Lwt.t
-  (** Same as {!read_byte} but does not raises [End_of_file] on end of
+  (** Same as {!read_byte} but does not raise [End_of_file] on end of
       input *)
 
 val read_chars : input_channel -> char Lwt_stream.t
@@ -252,7 +258,7 @@ val read : ?count : int -> input_channel -> string Lwt.t
       returns [""] if the end of input is reached. If [count] is not
       specified, it reads all bytes until the end of input. *)
 
-val read_into : input_channel -> string -> int -> int -> int Lwt.t
+val read_into : input_channel -> Bytes.t -> int -> int -> int Lwt.t
   (** [read_into ic buffer offset length] reads up to [length] bytes,
       stores them in [buffer] at offset [offset], and returns the
       number of bytes read.
@@ -260,7 +266,7 @@ val read_into : input_channel -> string -> int -> int -> int Lwt.t
       Note: [read_into] does not raise [End_of_file], it returns a
       length of [0] instead. *)
 
-val read_into_exactly : input_channel -> string -> int -> int -> unit Lwt.t
+val read_into_exactly : input_channel -> Bytes.t -> int -> int -> unit Lwt.t
   (** [read_into_exactly ic buffer offset length] reads exactly
       [length] bytes and stores them in [buffer] at offset [offset].
 
@@ -269,12 +275,12 @@ val read_into_exactly : input_channel -> string -> int -> int -> unit Lwt.t
 val read_value : input_channel -> 'a Lwt.t
   (** [read_value ic] reads a marshaled value from [ic] *)
 
-(** {6 Writing} *)
+(** {2 Writing} *)
 
 (** Note: as for reading functions, all functions except
     {!write_chars} and {!write_lines} are {b atomic}.
 
-    For example if you use {!write_line} in to different threads, the
+    For example if you use {!write_line} in two different threads, the
     two operations will be serialized, and lines cannot be mixed.
 *)
 
@@ -295,27 +301,33 @@ val write_line : output_channel -> string -> unit Lwt.t
 val write_lines : output_channel -> string Lwt_stream.t -> unit Lwt.t
   (** [write_lines oc lines] writes all lines of [lines] to [oc] *)
 
-val write_from : output_channel -> string -> int -> int -> int Lwt.t
+val write_from : output_channel -> Bytes.t -> int -> int -> int Lwt.t
   (** [write_from oc buffer offset length] writes up to [length] bytes
       to [oc], from [buffer] at offset [offset] and returns the number
       of bytes actually written *)
 
-val write_from_exactly : output_channel -> string -> int -> int -> unit Lwt.t
+val write_from_string : output_channel -> string -> int -> int -> int Lwt.t
+  (** See {!write}. *)
+
+val write_from_exactly : output_channel -> Bytes.t -> int -> int -> unit Lwt.t
   (** [write_from_exactly oc buffer offset length] writes all [length]
       bytes from [buffer] at offset [offset] to [oc] *)
 
+val write_from_string_exactly : output_channel -> string -> int -> int -> unit Lwt.t
+  (** See {!write_from_exactly}. *)
+
 val write_value : output_channel -> ?flags : Marshal.extern_flags list -> 'a -> unit Lwt.t
   (** [write_value oc ?flags x] marshals the value [x] to [oc] *)
 
-(** {6 Printing} *)
+(** {2 Printing} *)
 
-(** These functions are basically helpers. Also you may prefer the
+(** These functions are basically helpers. Also you may prefer
     using the name {!printl} rather than {!write_line} because it is
     shorter.
 
-    The general name of a printing function is [<prefix>print<suffixes>].
+    The general name of a printing function is [<prefix>print<suffixes>],
 
-    Where [<prefix>] is one of:
+    where [<prefix>] is one of:
     - ['f'], which means that the function takes as argument a channel
     - nothing, which means that the function prints on {!stdout}
     - ['e'], which means that the function prints on {!stderr}
@@ -339,7 +351,7 @@ val eprintl : string -> unit Lwt.t
 val eprintf : ('a, unit, string, unit Lwt.t) format4 -> 'a
 val eprintlf : ('a, unit, string, unit Lwt.t) format4 -> 'a
 
-(** {6 Utilities} *)
+(** {2 Utilities} *)
 
 val hexdump_stream : output_channel -> char Lwt_stream.t -> unit Lwt.t
   (** [hexdump_stream oc byte_stream] produces the same output as the
@@ -348,18 +360,18 @@ val hexdump_stream : output_channel -> char Lwt_stream.t -> unit Lwt.t
 val hexdump : output_channel -> string -> unit Lwt.t
   (** [hexdump oc str = hexdump_stream oc (Lwt_stream.of_string str)] *)
 
-(** {6 File utilities} *)
+(** {2 File utilities} *)
 
 type file_name = string
     (** Type of file names *)
 
 val open_file :
-  ?buffer_size : int ->
+  ?buffer : Lwt_bytes.t ->
   ?flags : Unix.open_flag list ->
   ?perm : Unix.file_perm ->
   mode : 'a mode ->
   file_name -> 'a channel Lwt.t
-  (** [open_file ?buffer_size ?flags ?perm ~mode filename] open the
+  (** [open_file ?buffer ?flags ?perm ~mode filename] opens the
       file with name [filename] and returns a channel for
       reading/writing it.
 
@@ -367,18 +379,22 @@ val open_file :
   *)
 
 val with_file :
-  ?buffer_size : int ->
+  ?buffer : Lwt_bytes.t ->
   ?flags : Unix.open_flag list ->
   ?perm : Unix.file_perm ->
   mode : 'a mode ->
   file_name -> ('a channel -> 'b Lwt.t) -> 'b Lwt.t
-  (** [with_file ?buffer_size ?flags ?perm ~mode filename f] open a
+  (** [with_file ?buffer ?flags ?perm ~mode filename f] opens a
       file and passes the channel to [f]. It is ensured that the
       channel is closed when [f ch] terminates (even if it fails). *)
 
-val open_connection : ?buffer_size : int -> Unix.sockaddr -> (input_channel * output_channel) Lwt.t
-  (** [open_connection ?buffer_size addr] open a connection to
-      the given address and returns two channels for using it.
+val open_connection :
+  ?fd : Lwt_unix.file_descr ->
+  ?in_buffer : Lwt_bytes.t -> ?out_buffer : Lwt_bytes.t ->
+  Unix.sockaddr -> (input_channel * output_channel) Lwt.t
+  (** [open_connection ?fd ?in_buffer ?out_buffer addr] opens a
+      connection to the given address and returns two channels for using
+      it. If [fd] is not specified, a fresh one will be used.
 
       The connection is completly closed when you close both
       channels.
@@ -386,18 +402,27 @@ val open_connection : ?buffer_size : int -> Unix.sockaddr -> (input_channel * ou
       @raise Unix.Unix_error on error.
   *)
 
-val with_connection : ?buffer_size : int -> Unix.sockaddr -> (input_channel * output_channel -> 'a Lwt.t) -> 'a Lwt.t
-  (** [with_connection ?buffer_size addr f] open a connection to
-      the given address and passes the channels to [f] *)
+val with_connection :
+  ?fd : Lwt_unix.file_descr ->
+  ?in_buffer : Lwt_bytes.t -> ?out_buffer : Lwt_bytes.t ->
+  Unix.sockaddr -> (input_channel * output_channel -> 'a Lwt.t) -> 'a Lwt.t
+  (** [with_connection ?fd ?in_buffer ?out_buffer addr f] opens a
+      connection to the given address and passes the channels to
+      [f] *)
 
 type server
   (** Type of a server *)
 
-val establish_server : ?buffer_size : int -> ?backlog : int -> Unix.sockaddr -> (input_channel * output_channel -> unit) -> server
-  (** [establich_server ?buffer_size ?backlog sockaddr f] creates a
-      server which will listen for incomming connections. New
+val establish_server :
+  ?fd : Lwt_unix.file_descr ->
+  ?buffer_size : int ->
+  ?backlog : int ->
+  Unix.sockaddr -> (input_channel * output_channel -> unit) -> server
+  (** [establish_server ?fd ?buffer_size ?backlog sockaddr f] creates
+      a server which will listen for incoming connections. New
       connections are passed to [f]. Note that [f] must not raise any
-      exception.
+      exception. If [fd] is not specified, a fresh file descriptor will
+      be created.
 
       [backlog] is the argument passed to [Lwt_unix.listen] *)
 
@@ -411,7 +436,7 @@ val lines_of_file : file_name -> string Lwt_stream.t
 
 val lines_to_file : file_name -> string Lwt_stream.t -> unit Lwt.t
   (** [lines_to_file name lines] writes all lines of [lines] to
-      [files] *)
+      file with name [name]. *)
 
 val chars_of_file : file_name -> char Lwt_stream.t
   (** [chars_of_file name] returns a stream of all characters of the
@@ -422,12 +447,12 @@ val chars_to_file : file_name -> char Lwt_stream.t -> unit Lwt.t
   (** [chars_to_file name chars] writes all characters of [chars] to
       [name] *)
 
-(** {6 Input/output of integers} *)
+(** {2 Input/output of integers} *)
 
 (** Common interface for reading/writing integers in binary *)
 module type NumberIO = sig
 
-  (** {8 Reading} *)
+  (** {3 Reading} *)
 
   val read_int : input_channel -> int Lwt.t
     (** Reads a 32-bits integer as an ocaml int *)
@@ -442,7 +467,7 @@ module type NumberIO = sig
   val read_float64 : input_channel -> float Lwt.t
     (** Reads an IEEE double precision floating point value *)
 
-  (** {8 Writing} *)
+  (** {3 Writing} *)
 
   val write_int : output_channel -> int -> unit Lwt.t
     (** Writes an ocaml int as a 32-bits integer *)
@@ -473,15 +498,15 @@ type byte_order = Lwt_sys.byte_order = Little_endian | Big_endian
 val system_byte_order : byte_order
   (** Same as {!Lwt_sys.byte_order}. *)
 
-(** {6 Low-level access to the internal buffer} *)
+(** {2 Low-level access to the internal buffer} *)
 
 val block : 'a channel  -> int -> (Lwt_bytes.t -> int -> 'b Lwt.t) -> 'b Lwt.t
   (** [block ch size f] pass to [f] the internal buffer and an
       offset. The buffer contains [size] chars at [offset]. [f] may
-      reads or writes these chars.  [size] must verify [0 <= size <=
+      read or write these chars.  [size] must satisfy [0 <= size <=
       16] *)
 
-(** Informations for accessing directly to the internal buffer of a
+(** Information for directly accessing the internal buffer of a
     channel *)
 type direct_access = {
   da_buffer : Lwt_bytes.t;
@@ -494,29 +519,24 @@ type direct_access = {
   (** The maximum offset *)
   da_perform : unit -> int Lwt.t;
   (** - for input channels:
-        refill the buffer and returns how many bytes have been read
+        refills the buffer and returns how many bytes have been read
       - for output channels:
         flush partially the buffer and returns how many bytes have been written *)
 }
 
 val direct_access : 'a channel -> (direct_access -> 'b Lwt.t) -> 'b Lwt.t
-  (** [direct_access ch f] pass to [f] a {!direct_access}
+  (** [direct_access ch f] passes to [f] a {!direct_access}
       structure. [f] must use it and update [da_ptr] to reflect how
       many bytes have been read/written. *)
 
-(** {6 Misc} *)
+(** {2 Misc} *)
 
 val default_buffer_size : unit -> int
   (** Return the default size for buffers. Channels that are created
-      without specific size use this one. *)
+      without a specific buffer use new buffer of this size. *)
 
 val set_default_buffer_size : int -> unit
   (** Change the default buffer size.
 
       @raise Invalid_argument if the given size is smaller than [16]
       or greater than [Sys.max_string_length] *)
-
-(**/**)
-
-val of_string : mode : 'mode mode -> string -> 'mode channel
-  (* Deprecated *)
