@@ -1,4 +1,4 @@
-/* Lightweight thread library for Objective Caml
+/* Lightweight thread library for OCaml
  * http://www.ocsigen.org/lwt
  * Module Lwt_unix_stubs
  * Copyright (C) 2010 Jérémie Dimino
@@ -23,6 +23,7 @@
 /* Stubs for libev */
 
 #include "lwt_config.h"
+#include "lwt_unix.h"
 
 #if defined(HAVE_LIBEV)
 
@@ -35,8 +36,6 @@
 #include <caml/callback.h>
 #include <ev.h>
 
-#include "lwt_unix.h"
-
 /* +-----------------------------------------------------------------+
    | Loops                                                           |
    +-----------------------------------------------------------------+ */
@@ -71,7 +70,7 @@ static void nop(struct ev_loop *loop)
 {
 }
 
-CAMLprim value lwt_libev_init()
+CAMLprim value lwt_libev_init(value Unit)
 {
   struct ev_loop *loop = ev_loop_new(EVFLAG_FORKCHECK);
   if (!loop) caml_failwith("lwt_libev_init");
@@ -194,7 +193,11 @@ CAMLprim value lwt_libev_timer_init(value loop, value delay, value repeat, value
   CAMLlocal1(result);
   /* Create and initialise the watcher */
   struct ev_timer* watcher = lwt_unix_new(struct ev_timer);
-  ev_timer_init(watcher, handle_timer, Double_val(delay), Bool_val(repeat));
+  if (Bool_val(repeat))
+    ev_timer_init(watcher, handle_timer, Double_val(delay), Double_val(delay));
+  else
+    ev_timer_init(watcher, handle_timer, Double_val(delay), 0.0);
+
   /* Wrap the watcher into a custom caml value */
   result = caml_alloc_custom(&watcher_ops, sizeof(struct ev_timer*), 0, 1);
   Ev_timer_val(result) = watcher;
@@ -216,4 +219,16 @@ CAMLprim value lwt_libev_timer_stop(value loop, value val_watcher)
   CAMLreturn(Val_unit);
 }
 
+#else
+
+LWT_NOT_AVAILABLE1(libev_init)
+LWT_NOT_AVAILABLE1(libev_stop)
+LWT_NOT_AVAILABLE2(libev_loop)
+LWT_NOT_AVAILABLE1(libev_unloop)
+LWT_NOT_AVAILABLE3(libev_readable_init)
+LWT_NOT_AVAILABLE3(libev_writable_init)
+LWT_NOT_AVAILABLE2(libev_io_stop)
+LWT_NOT_AVAILABLE4(libev_timer_init)
+LWT_NOT_AVAILABLE2(libev_timer_stop)
+
 #endif
