@@ -1,9 +1,9 @@
-(* Lightweight thread library for Objective Caml
+(* Lightweight thread library for OCaml
  * http://www.ocsigen.org/lwt
  * Interface Lwt_unix
- * Copyright (C) 2005-2008 Jrme Vouillon
- * Laboratoire PPS - CNRS Universit Paris Diderot
- *                    2009 Jrmie Dimino
+ * Copyright (C) 2005-2008 Jérôme Vouillon
+ * Laboratoire PPS - CNRS Université Paris Diderot
+ *                    2009 Jérémie Dimino
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as
@@ -24,44 +24,44 @@
 
 (** Cooperative system calls *)
 
-(** This modules redefine system calls, as in the [Unix] module of the
-    standard library, but mapped into cooperative ones, which will not
-    block the program, letting other threads run.
+(** This modules maps system calls, like those of the standard
+    library's [Unix] module, to cooperative ones, which will not block
+    the program.
 
-    The semantic of all operations is the following: if the action
+    The semantics of all operations is the following: if the action
     (for example reading from a {b file descriptor}) can be performed
-    immediatly, it is done and returns immediatly, otherwise it
-    returns a sleeping threads which is waked up when the operation
+    immediately, it is done and returns immediately, otherwise it
+    returns a sleeping thread which is woken up when the operation
     completes.
 
     Most operations on sockets and pipes (on Windows it is only
-    sockets) are {b cancelable}, this means that you can cancel them
+    sockets) are {b cancelable}, meaning you can cancel them
     with {!Lwt.cancel}. For example if you want to read something from
     a {b file descriptor} with a timeout, you can cancel the action
     after the timeout and the reading will not be performed if not
     already done.
 
-    More precisely, assuming that you have two sockets [sock1] and
-    [sock2] and you want to read something from [sock1] or exclusively
-    from [sock2], and fail with an exception if a timeout of 1 second
+    For example, consider that you have two sockets [sock1] and
+    [sock2]. You want to read something from [sock1] or exclusively
+    from [sock2] and fail with an exception if a timeout of 1 second
     expires, without reading anything from [sock1] and [sock2], even
     if they become readable in the future.
 
     Then you can do:
 
     {[
-      Lwt.pick [Lwt_unix.timeout 1.0; read sock1 buf1 ofs1 len1; read sock2 buf2 ofs2 len2]
+    Lwt.pick [Lwt_unix.timeout 1.0; read sock1 buf1 ofs1 len1; read sock2 buf2 ofs2 len2]
     ]}
 
-    In this case it is guaranteed that exactly one of the three
-    operations will completes, and other will just be cancelled.
+    In this case, it is guaranteed that exactly one of the three
+    operations will complete, and the others will be cancelled.
 *)
 
 val handle_unix_error : ('a -> 'b Lwt.t) -> 'a -> 'b Lwt.t
   (** Same as [Unix.handle_unix_error] but catches lwt-level
       exceptions *)
 
-(** {6 Configuration} *)
+(** {2 Configuration} *)
 
 (** For system calls that cannot be made asynchronously, Lwt uses one
     of the following method: *)
@@ -72,13 +72,13 @@ type async_method =
   | Async_detach
       (** System calls are made in another system thread, thus without
           blocking other Lwt threads. The drawback is that it may
-          degrade performances in some cases.
+          degrade performance in some cases.
 
           This is the default. *)
   | Async_switch
       (** System calls are made in the main thread, and if one blocks
           the execution continue in another system thread. This method
-          is the most efficint, also you will get better performances
+          is the most efficient, also you will get better performance
           if you force all threads to run on the same cpu. On linux
           this can be done by using the command [taskset].
 
@@ -125,26 +125,26 @@ val with_async_switch : (unit -> 'a) -> 'a
       ]}
   *)
 
-(** {6 Sleeping} *)
+(** {2 Sleeping} *)
 
 val sleep : float -> unit Lwt.t
-  (** [sleep d] is a threads which remain suspended for [d] seconds
+  (** [sleep d] is a thread that remains suspended for [d] seconds
       and then terminates. *)
 
 val yield : unit -> unit Lwt.t
-  (** [yield ()] is a threads which suspends itself and then resumes
+  (** [yield ()] is a thread that suspends itself and then resumes
       as soon as possible and terminates. *)
 
 val auto_yield : float -> (unit -> unit Lwt.t)
-  (** [auto_yield timeout] returns a function [f] which will yield
+  (** [auto_yield timeout] returns a function [f] that will yield
       every [timeout] seconds. *)
 
 exception Timeout
   (** Exception raised by timeout operations *)
 
 val timeout : float -> 'a Lwt.t
-  (** [timeout d] is a thread which remains suspended for [d] seconds
-      then fails with {!Timeout} *)
+  (** [timeout d] is a thread that remains suspended for [d] seconds
+      and then fails with {!Timeout}. *)
 
 val with_timeout : float -> (unit -> 'a Lwt.t) -> 'a Lwt.t
   (** [with_timeout d f] is a short-hand for:
@@ -154,7 +154,7 @@ val with_timeout : float -> (unit -> 'a Lwt.t) -> 'a Lwt.t
       ]}
   *)
 
-(** {6 Operation on file-descriptors} *)
+(** {2 Operation on file-descriptors} *)
 
 type file_descr
   (** The abstract type for {b file descriptor}s. A Lwt {b file
@@ -228,7 +228,7 @@ val abort : file_descr -> exn -> unit
       Note that this only works for reading and writing operations on
       file descriptors supporting non-blocking mode. *)
 
-(** {6 Process handling} *)
+(** {2 Process handling} *)
 
 val fork : unit -> int
   (** [fork ()] does the same as [Unix.fork]. You must use this
@@ -284,7 +284,7 @@ val system : string -> process_status Lwt.t
       shell [/bin/sh] on Unix and [cmd.exe] on Windows. The result
       [WEXITED 127] indicates that the shell couldn't be executed. *)
 
-(** {6 Basic file input/output} *)
+(** {2 Basic file input/output} *)
 
 val stdin : file_descr
   (** The standard {b file descriptor} for input. This one is usually
@@ -312,9 +312,8 @@ type open_flag =
   | O_DSYNC
   | O_SYNC
   | O_RSYNC
-#if ocaml_version >= (3, 13)
   | O_SHARE_DELETE
-#endif
+  | O_CLOEXEC
 
 val openfile : string -> open_flag list -> file_perm -> file_descr Lwt.t
   (** Wrapper for [Unix.openfile]. *)
@@ -323,14 +322,17 @@ val close : file_descr -> unit Lwt.t
   (** Close a {b file descriptor}. This close the underlying unix {b
       file descriptor} and set its state to {!Closed} *)
 
-val read : file_descr -> string -> int -> int -> int Lwt.t
+val read : file_descr -> Bytes.t -> int -> int -> int Lwt.t
   (** [read fd buf ofs len] has the same semantic as [Unix.read], but
       is cooperative *)
 
-val write : file_descr -> string -> int -> int -> int Lwt.t
-  (** [read fd buf ofs len] has the same semantic as [Unix.write], but
+val write : file_descr -> Bytes.t -> int -> int -> int Lwt.t
+  (** [write fd buf ofs len] has the same semantic as [Unix.write], but
       is cooperative *)
 
+val write_string : file_descr -> string -> int -> int -> int Lwt.t
+  (** See {!write}. *)
+
 val readable : file_descr -> bool
   (** Returns whether the given file descriptor is currently
       readable. *)
@@ -347,7 +349,7 @@ val wait_write : file_descr -> unit Lwt.t
   (** waits (without blocking other threads) until it is possible to
       write on the file descriptor *)
 
-(** {6 Seeking and truncating} *)
+(** {2 Seeking and truncating} *)
 
 type seek_command =
     Unix.seek_command =
@@ -364,7 +366,7 @@ val truncate : string -> int -> unit Lwt.t
 val ftruncate : file_descr -> int -> unit Lwt.t
   (** Wrapper for [Unix.ftruncate] *)
 
-(** {6 Syncing} *)
+(** {2 Syncing} *)
 
 val fsync : file_descr -> unit Lwt.t
   (** Synchronise all data and metadata of the file descriptor with
@@ -376,7 +378,7 @@ val fdatasync : file_descr -> unit Lwt.t
 
       Note that [fdatasync] is not available on all platforms. *)
 
-(** {6 File status} *)
+(** {2 File status} *)
 
 type file_kind =
     Unix.file_kind =
@@ -417,7 +419,7 @@ val fstat : file_descr -> stats Lwt.t
 val isatty : file_descr -> bool Lwt.t
   (** Wrapper for [Unix.isatty] *)
 
-(** {6 File operations on large files} *)
+(** {2 File operations on large files} *)
 
 module LargeFile : sig
   val lseek : file_descr -> int64 -> seek_command -> int64 Lwt.t
@@ -456,7 +458,7 @@ module LargeFile : sig
     (** Wrapper for [Unix.LargeFile.fstat] *)
 end
 
-(** {6 Operations on file names} *)
+(** {2 Operations on file names} *)
 
 val unlink : string -> unit Lwt.t
   (** Wrapper for [Unix.unlink] *)
@@ -467,7 +469,7 @@ val rename : string -> string -> unit Lwt.t
 val link : string -> string -> unit Lwt.t
   (** Wrapper for [Unix.link] *)
 
-(** {6 File permissions and ownership} *)
+(** {2 File permissions and ownership} *)
 
 val chmod : string -> file_perm -> unit Lwt.t
   (** Wrapper for [Unix.chmod] *)
@@ -491,7 +493,7 @@ type access_permission =
 val access : string -> access_permission list -> unit Lwt.t
   (** Wrapper for [Unix.access] *)
 
-(** {6 Operations on file descriptors} *)
+(** {2 Operations on file descriptors} *)
 
 val dup : file_descr -> file_descr
   (** Wrapper for [Unix.dup] *)
@@ -505,7 +507,7 @@ val set_close_on_exec : file_descr -> unit
 val clear_close_on_exec : file_descr -> unit
   (** Wrapper for [Unix.clear_close_on_exec] *)
 
-(** {6 Directories} *)
+(** {2 Directories} *)
 
 val mkdir : string -> file_perm -> unit Lwt.t
   (** Wrapper for [Unix.mkdir] *)
@@ -525,7 +527,7 @@ val opendir : string -> dir_handle Lwt.t
   (** Wrapper for [Unix.opendir] *)
 
 val readdir : dir_handle -> string Lwt.t
-  (** Wrapper for [Unix.dir] *)
+  (** Wrapper for [Unix.readdir]. *)
 
 val readdir_n : dir_handle -> int -> string array Lwt.t
   (** [readdir_n handle count] reads at most [count] entry from the
@@ -544,7 +546,7 @@ val files_of_directory : string -> string Lwt_stream.t
   (** [files_of_directory dir] returns the stream of all files of
       [dir]. *)
 
-(** {6 Pipes and redirections} *)
+(** {2 Pipes and redirections} *)
 
 val pipe : unit -> file_descr * file_descr
   (** [pipe ()] creates pipe using [Unix.pipe] and returns two lwt {b
@@ -563,7 +565,7 @@ val pipe_out : unit -> Unix.file_descr * file_descr
 val mkfifo : string -> file_perm -> unit Lwt.t
   (** Wrapper for [Unix.mkfifo] *)
 
-(** {6 Symbolic links} *)
+(** {2 Symbolic links} *)
 
 val symlink : string -> string -> unit Lwt.t
   (** Wrapper for [Unix.symlink] *)
@@ -571,7 +573,7 @@ val symlink : string -> string -> unit Lwt.t
 val readlink : string -> string Lwt.t
   (** Wrapper for [Unix.readlink] *)
 
-(** {6 Locking} *)
+(** {2 Locking} *)
 
 type lock_command =
     Unix.lock_command =
@@ -585,7 +587,7 @@ type lock_command =
 val lockf : file_descr -> lock_command -> int -> unit Lwt.t
   (** Wrapper for [Unix.lockf] *)
 
-(** {6 User id, group id} *)
+(** {2 User id, group id} *)
 
 type passwd_entry =
     Unix.passwd_entry =
@@ -623,7 +625,7 @@ val getpwuid : int -> passwd_entry Lwt.t
 val getgrgid : int -> group_entry Lwt.t
   (** Wrapper for [Unix.getgrgid] *)
 
-(** {6 Signals} *)
+(** {2 Signals} *)
 
 type signal_handler_id
   (** Id of a signal handler, used to cancel it *)
@@ -631,7 +633,7 @@ type signal_handler_id
 val on_signal : int -> (int -> unit) -> signal_handler_id
   (** [on_signal signum f] calls [f] each time the signal with numnber
       [signum] is received by the process. It returns a signal handler
-      identifier which can be used to stop monitoring [signum]. *)
+      identifier that can be used to stop monitoring [signum]. *)
 
 val on_signal_full : int -> (signal_handler_id -> int -> unit) -> signal_handler_id
   (** [on_signal_full f] is the same as [on_signal f] except that [f]
@@ -650,7 +652,7 @@ val reinstall_signal_handler : int -> unit
       signal handler (with [Sys.set_signal]). This is usefull in case
       another part of the program install another signal handler. *)
 
-(** {6 Sockets} *)
+(** {2 Sockets} *)
 
 type inet_addr = Unix.inet_addr
 
@@ -686,21 +688,21 @@ val accept : file_descr -> (file_descr * sockaddr) Lwt.t
   (** Wrapper for [Unix.accept] *)
 
 val accept_n : file_descr -> int -> ((file_descr * sockaddr) list * exn option) Lwt.t
-  (** [accept_n fd count] accepts up to [count] connection in one time.
+  (** [accept_n fd count] accepts up to [count] connections at one time.
 
       - if no connection is available right now, it returns a sleeping
       thread
 
-      - if more that 1 and less than [count] are available, it returns
+      - if more than 1 and less than [count] are available, it returns
       all of them
 
-      - if more that [count] are available, it returns the next
+      - if more than [count] are available, it returns the next
       [count] of them
 
-      - if an error happen, it returns the connections that have been
+      - if an error happens, it returns the connections that have been
       successfully accepted so far and the error
 
-      [accept_n] has the advantage of improving performances. If you
+      [accept_n] has the advantage of improving performance. If you
       want a more detailed description, you can have a look at:
 
       {{:http://portal.acm.org/citation.cfm?id=1247435}Acceptable strategies for improving web server performance} *)
@@ -729,16 +731,16 @@ type msg_flag =
   | MSG_DONTROUTE
   | MSG_PEEK
 
-val recv : file_descr -> string -> int -> int -> msg_flag list -> int Lwt.t
+val recv : file_descr -> Bytes.t -> int -> int -> msg_flag list -> int Lwt.t
   (** Wrapper for [Unix.recv] *)
 
-val recvfrom : file_descr -> string -> int -> int -> msg_flag list -> (int * sockaddr) Lwt.t
+val recvfrom : file_descr -> Bytes.t -> int -> int -> msg_flag list -> (int * sockaddr) Lwt.t
   (** Wrapper for [Unix.recvfrom] *)
 
-val send : file_descr -> string -> int -> int -> msg_flag list -> int Lwt.t
+val send : file_descr -> Bytes.t -> int -> int -> msg_flag list -> int Lwt.t
   (** Wrapper for [Unix.send] *)
 
-val sendto : file_descr -> string -> int -> int -> msg_flag list -> sockaddr -> int Lwt.t
+val sendto : file_descr -> Bytes.t -> int -> int -> msg_flag list -> sockaddr -> int Lwt.t
   (** Wrapper for [Unix.sendto] *)
 
 (** An io-vector. Used by {!recv_msg} and {!send_msg}. *)
@@ -752,19 +754,23 @@ val io_vector : buffer : string -> offset : int -> length : int -> io_vector
   (** Creates an io-vector *)
 
 val recv_msg : socket : file_descr -> io_vectors : io_vector list -> (int * Unix.file_descr list) Lwt.t
-  (** [recv_msg ~socket ~io_vectors] receives data into a list of
-      io-vectors, plus any file-descriptors that may accompany the
-      message.
+(** [recv_msg ~socket ~io_vectors] receives data into a list of
+    io-vectors, plus any file-descriptors that may accompany the
+    messages. It returns a tuple whose first field is the number of
+    bytes received and second is a list of received file
+    descriptors. The messages themselves will be recorded in the
+    provided [io_vectors] list.
 
-      This call is not available on windows. *)
+    This call is not available on windows. *)
 
 val send_msg : socket : file_descr -> io_vectors : io_vector list -> fds : Unix.file_descr list -> int Lwt.t
-  (** [send_msg ~socket ~io_vectors ~fds] sends data from a list of
-      io-vectors, accompanied with a list of file-descriptor. If
-      fd-passing is not possible on the current system and [fds] is
-      not empty, it raises [Lwt_sys.Not_available "fd_passing"].
+(** [send_msg ~socket ~io_vectors ~fds] sends data from a list of
+    io-vectors, accompanied with a list of file-descriptors. It
+    returns the number of bytes sent. If fd-passing is not possible on
+    the current system and [fds] is not empty, it raises
+    [Lwt_sys.Not_available "fd_passing"].
 
-      This call is not available on windows. *)
+    This call is not available on windows. *)
 
 type credentials = {
   cred_pid : int;
@@ -773,15 +779,15 @@ type credentials = {
 }
 
 val get_credentials : file_descr -> credentials
-  (** [get_credentials fd] returns credential informations from the
+  (** [get_credentials fd] returns credentials information from the
       given socket. On some platforms, obtaining the peer pid is not
-      possible and it will be set to [-1]. If obtaining credentials 
+      possible and it will be set to [-1]. If obtaining credentials
       is not possible on the current system, it raises
       [Lwt_sys.Not_available "get_credentials"].
 
       This call is not available on windows. *)
 
-(** {8 Socket options} *)
+(** {3 Socket options} *)
 
 type socket_bool_option =
     Unix.socket_bool_option =
@@ -838,7 +844,23 @@ val setsockopt_float : file_descr -> socket_float_option -> float -> unit
 val getsockopt_error : file_descr -> Unix.error option
   (** Wrapper for [Unix.getsockopt_error] *)
 
-(** {6 Host and protocol databases} *)
+(** {3 Multicast functions} *)
+
+val mcast_set_loop : file_descr -> bool -> unit
+  (** Whether sent multicast messages are received by the sending host *)
+
+val mcast_set_ttl : file_descr -> int -> unit
+  (** Set TTL/hops value *)
+
+val mcast_add_membership : file_descr -> ?ifname:Unix.inet_addr -> Unix.inet_addr -> unit
+  (** [mcast_add_membership fd ~ifname addr] joins the multicast group [addr]
+      on the network interface [ifname]. *)
+
+val mcast_drop_membership : file_descr -> ?ifname:Unix.inet_addr -> Unix.inet_addr -> unit
+  (** [mcast_drop_membership fd ~ifname addr] leaves the multicast group [addr]
+      on the network interface [ifname]. *)
+
+(** {2 Host and protocol databases} *)
 
 type host_entry =
     Unix.host_entry =
@@ -927,7 +949,7 @@ type getnameinfo_option =
 val getnameinfo : sockaddr -> getnameinfo_option list -> name_info Lwt.t
   (** Wrapper for [Unix.getnameinfo] *)
 
-(** {6 Terminal interface} *)
+(** {2 Terminal interface} *)
 
 type terminal_io =
     Unix.terminal_io =
@@ -1009,7 +1031,7 @@ type flow_action =
 val tcflow : file_descr -> flow_action -> unit Lwt.t
   (** Wrapper for [Unix.tcflow] *)
 
-(** {6 Low-level interaction} *)
+(** {2 Low-level interaction} *)
 
 exception Retry
   (** If an action raises {!Retry}, it will be requeued until the {b
@@ -1027,9 +1049,9 @@ type io_event = Read | Write
 
 val wrap_syscall : io_event -> file_descr -> (unit -> 'a) -> 'a Lwt.t
   (** [wrap_syscall set fd action] wrap an action on a {b file
-      descriptor}. It tries to execture action, and if it can not be
+      descriptor}. It tries to execute action, and if it can not be
       performed immediately without blocking, it is registered for
-      latter.
+      later.
 
       In the latter case, if the thread is canceled, [action] is
       removed from [set]. *)
@@ -1099,7 +1121,7 @@ val cancel_jobs : unit -> unit
 val wait_for_jobs : unit -> unit Lwt.t
   (** Wait for all pending jobs to terminate. *)
 
-(** {6 Notifications} *)
+(** {2 Notifications} *)
 
 (** Lwt internally use a pipe to send notification to the main
     thread. The following functions allow to use this pipe. *)
@@ -1131,7 +1153,7 @@ val set_notification : int -> (unit -> unit) -> unit
       notification by [f]. It raises [Not_found] if the given
       notification is not found. *)
 
-(** {6 System threads pool} *)
+(** {2 System threads pool} *)
 
 (** If the program is using the async method {!Async_detach} or
     {!Async_switch}, Lwt will launch system threads to execute
@@ -1150,7 +1172,7 @@ val thread_count : unit -> int
 val thread_waiting_count : unit -> int
   (** The number threads waiting for a job. *)
 
-(** {6 CPUs} *)
+(** {2 CPUs} *)
 
 val get_cpu : unit -> int
   (** [get_cpu ()] returns the number of the CPU the current thread is
