@@ -1,4 +1,4 @@
-/* Lightweight thread library for Objective Caml
+/* Lightweight thread library for OCaml
  * http://www.ocsigen.org/lwt
  * Module Lwt_unix_unix
  * Copyright (C) 2009-2010 Jérémie Dimino
@@ -80,7 +80,7 @@ CAMLprim value lwt_unix_madvise (value val_buffer, value val_offset, value val_l
   return Val_unit;
 }
 
-CAMLprim value lwt_unix_get_page_size()
+CAMLprim value lwt_unix_get_page_size(value Unit)
 {
   long page_size = sysconf(_SC_PAGESIZE);
   if (page_size < 0) page_size = 4096;
@@ -468,8 +468,139 @@ CAMLprim value lwt_unix_get_credentials(value fd)
     CAMLreturn(res);
 }
 
+#else
+
+LWT_NOT_AVAILABLE1(unix_get_credentials)
+
 #endif
 
+/* +-----------------------------------------------------------------+
+   | Multicast functions                                             |
+   +-----------------------------------------------------------------+ */
+
+static int socket_domain (int fd)
+{
+    /* Return the socket domain, PF_INET or PF_INET6. Fails for non-IP
+       protos.
+       fd must be a socket!
+    */
+    union sock_addr_union addr;
+    socklen_t l;
+
+    l = sizeof(addr);
+    if (getsockname(fd, &addr.s_gen, &l) == -1)
+        uerror("getsockname", Nothing);
+
+    switch (addr.s_gen.sa_family) {
+    case AF_INET:
+        return PF_INET;
+    case AF_INET6:
+        return PF_INET6;
+    default:
+        invalid_argument("Not an Internet socket");
+    }
+
+    return 0;
+}
+
+
+CAMLprim value lwt_unix_mcast_set_loop (value fd, value flag)
+{
+    int t, r, f;
+
+    t = socket_domain(Int_val(fd));
+    f = Bool_val(flag);
+    r = 0;
+
+    switch (t) {
+    case PF_INET:
+        r = setsockopt (Int_val(fd), IPPROTO_IP, IP_MULTICAST_LOOP, (void *) &f, sizeof(f));
+        break;
+    default:
+        invalid_argument("lwt_unix_mcast_set_loop");
+    };
+
+    if (r == -1)
+        uerror("setsockopt",Nothing);
+
+    return Val_unit;
+}
+
+
+CAMLprim value lwt_unix_mcast_set_ttl (value fd, value ttl)
+{
+    int t, r, v;
+    int fd_sock;
+
+    fd_sock = Int_val(fd);
+    t = socket_domain(fd_sock);
+    v = Int_val(ttl);
+    r = 0;
+
+    switch (t) {
+    case PF_INET:
+        r = setsockopt(fd_sock, IPPROTO_IP, IP_MULTICAST_TTL, (void *) &v, sizeof(v));
+        break;
+    default:
+        invalid_argument("lwt_unix_mcast_set_ttl");
+    };
+
+    if (r == -1)
+        uerror("setsockopt",Nothing);
+
+    return Val_unit;
+}
+
+/* Keep this in sync with the type Lwt_unix.mcast_action */
+#define VAL_MCAST_ACTION_ADD  (Val_int(0))
+#define VAL_MCAST_ACTION_DROP (Val_int(1))
+
+#define GET_INET_ADDR(v)      (*((struct in_addr *) (v)))
+
+CAMLprim value lwt_unix_mcast_modify_membership (value fd, value v_action, value if_addr, value group_addr)
+{
+    int t, r;
+    int fd_sock;
+    int optname;
+
+    fd_sock = Int_val(fd);
+    t = socket_domain(fd_sock);
+    r = 0;
+
+    switch (t) {
+    case PF_INET: {
+        struct ip_mreq mreq;
+
+        if (string_length(group_addr) != 4 || string_length(if_addr) != 4 )
+            invalid_argument("lwt_unix_mcast_modify: Not an IPV4 address");
+
+        memcpy(&mreq.imr_multiaddr, &GET_INET_ADDR(group_addr), 4);
+        memcpy(&mreq.imr_interface, &GET_INET_ADDR(if_addr), 4);
+
+        switch (v_action) {
+        case VAL_MCAST_ACTION_ADD:
+            optname = IP_ADD_MEMBERSHIP;
+            break;
+
+        default:
+            optname = IP_DROP_MEMBERSHIP;
+            break;
+        }
+
+        r = setsockopt(fd_sock, IPPROTO_IP, optname, (void *) &mreq, sizeof(mreq));
+        break;
+    }
+    default:
+        invalid_argument("lwt_unix_mcast_modify_membership");
+    };
+
+    if (r == -1)
+        uerror("setsockopt", Nothing);
+
+    return Val_unit;
+}
+
+
 /* +-----------------------------------------------------------------+
    | wait4                                                           |
    +-----------------------------------------------------------------+ */
@@ -479,6 +610,8 @@ CAMLprim value lwt_unix_get_credentials(value fd)
 CAMLextern int caml_convert_signal_number (int);
 CAMLextern int caml_rev_convert_signal_number (int);
 
+#if !defined(__ANDROID__)
+
 #if !(defined(WIFEXITED) && defined(WEXITSTATUS) && defined(WIFSTOPPED) && \
       defined(WSTOPSIG) && defined(WTERMSIG))
 /* Assume old-style V7 status word */
@@ -542,10 +675,11 @@ value lwt_unix_wait4(value flags, value pid_req)
   CAMLreturn(res);
 }
 
-value lwt_unix_has_wait4(value unit)
-{
-  return Val_int(1);
-}
+#else
+
+LWT_NOT_AVAILABLE2(unix_wait4)
+
+#endif
 
 /* +-----------------------------------------------------------------+
    | CPUs                                                            |
@@ -553,13 +687,17 @@ value lwt_unix_has_wait4(value unit)
 
 #if defined(HAVE_GETCPU)
 
-CAMLprim value lwt_unix_get_cpu()
+CAMLprim value lwt_unix_get_cpu(value Unit)
 {
   int cpu = sched_getcpu();
   if (cpu < 0) uerror("sched_getcpu", Nothing);
   return Val_int(cpu);
 }
 
+#else
+
+LWT_NOT_AVAILABLE1(unix_get_cpu)
+
 #endif
 
 #if defined(HAVE_AFFINITY)
@@ -595,6 +733,11 @@ CAMLprim value lwt_unix_set_affinity(value val_pid, value val_cpus)
   return Val_unit;
 }
 
+#else
+
+LWT_NOT_AVAILABLE1(unix_get_affinity)
+LWT_NOT_AVAILABLE2(unix_set_affinity)
+
 #endif
 
 /* +-----------------------------------------------------------------+
@@ -927,9 +1070,9 @@ static value copy_stat(int use_64, struct stat *buf)
   CAMLparam0();
   CAMLlocal5(atime, mtime, ctime, offset, v);
 
-  atime = copy_double((double) buf->st_atime);
-  mtime = copy_double((double) buf->st_mtime);
-  ctime = copy_double((double) buf->st_ctime);
+  atime = copy_double((double) buf->st_atime + (NANOSEC(buf, a) / 1000000000.0));
+  mtime = copy_double((double) buf->st_mtime + (NANOSEC(buf, m) / 1000000000.0));
+  ctime = copy_double((double) buf->st_ctime + (NANOSEC(buf, c) / 1000000000.0));
   offset = use_64 ? caml_copy_int64(buf->st_size) : Val_int(buf->st_size);
   v = alloc_small(12, 0);
   Field(v, 0) = Val_int (buf->st_dev);
@@ -1384,7 +1527,7 @@ static value result_readlink(struct job_readlink *job)
 {
   LWT_UNIX_CHECK_JOB_ARG(job, job->result < 0, "readlink", job->name);
   value result = caml_copy_string(job->buffer);
-  free(&job->buffer);
+  free(job->buffer);
   lwt_unix_free_job(&job->job);
   return result;
 }
@@ -1500,6 +1643,8 @@ CAMLprim value lwt_unix_lockf_job(value val_fd, value val_command, value val_len
    | JOB: getlogin                                                   |
    +-----------------------------------------------------------------+ */
 
+#if !defined(__ANDROID__)
+
 struct job_getlogin {
   struct lwt_unix_job job;
   char buffer[1024];
@@ -1524,16 +1669,24 @@ static value result_getlogin(struct job_getlogin *job)
   }
 }
 
-CAMLprim value lwt_unix_getlogin_job()
+CAMLprim value lwt_unix_getlogin_job(value Unit)
 {
   LWT_UNIX_INIT_JOB(job, getlogin, 0);
   return lwt_unix_alloc_job(&job->job);
 }
 
+#else
+
+LWT_NOT_AVAILABLE1(unix_getlogin_job)
+
+#endif
+
 /* +-----------------------------------------------------------------+
    | JOBs: get{pw,gr}{nam,uid}                                       |
    +-----------------------------------------------------------------+ */
 
+#if !defined(__ANDROID__)
+
 static value alloc_passwd_entry(struct passwd *entry)
 {
   value res;
@@ -1543,7 +1696,7 @@ static value alloc_passwd_entry(struct passwd *entry)
   Begin_roots5 (name, passwd, gecos, dir, shell);
     name = copy_string(entry->pw_name);
     passwd = copy_string(entry->pw_passwd);
-#ifndef __BEOS__
+#if !defined(__BEOS__)
     gecos = copy_string(entry->pw_gecos);
 #else
     gecos = copy_string("");
@@ -1629,6 +1782,112 @@ JOB_GET_ENTRY(LWT_UNIX_INIT_JOB_STRING(job, getgrnam, 0, name), getgrnam, GETGR,
 JOB_GET_ENTRY(LWT_UNIX_INIT_JOB(job, getpwuid, 0); job->uid = Int_val(uid), getpwuid, GETPW, passwd, uid, int uid, Nothing)
 JOB_GET_ENTRY(LWT_UNIX_INIT_JOB(job, getgrgid, 0); job->gid = Int_val(gid), getgrgid, GETGR, group, gid, int gid, Nothing)
 
+#else
+
+LWT_NOT_AVAILABLE1(unix_getpwnam_job)
+LWT_NOT_AVAILABLE1(unix_getgrnam_job)
+LWT_NOT_AVAILABLE1(unix_getpwuid_job)
+LWT_NOT_AVAILABLE1(unix_getgrgid_job)
+
+#endif
+
+/* Helper functions for not re-entrant functions */
+
+/* keep test in sync with discover.ml */
+#if !defined(HAS_GETHOSTBYADDR_R) || (HAS_GETHOSTBYADDR_R != 7 && HAS_GETHOSTBYADDR_R != 8)
+#define NON_R_GETHOSTBYADDR 1
+#endif
+
+/* keep test in sync with discover.ml */
+#if !defined(HAS_GETHOSTBYNAME_R) || (HAS_GETHOSTBYNAME_R != 5 && HAS_GETHOSTBYNAME_R != 6)
+#define NON_R_GETHOSTBYNAME 1
+#endif
+
+#if defined(NON_R_GETHOSTBYADDR) || defined(NON_R_GETHOSTBYNAME)
+static char **
+c_copy_addr_array(char ** src, int addr_len)
+{
+  if ( src == NULL ){
+    return NULL;
+  }
+  char ** p = src;
+  size_t i = 0 ;
+  while ( *p ){
+    i++;
+    p++;
+  }
+  const size_t ar_len = i;
+  p = malloc((ar_len+1) * sizeof(char*));
+  if ( p == NULL ){
+    return NULL;
+  }
+  for ( i = 0 ; i < ar_len ; ++i ){
+    p[i] = malloc(addr_len);
+    if ( p[i] == NULL ){
+      size_t j;
+      for ( j = 0 ; j < i ; j++ ){
+        free(p[j]);
+      }
+      free(p);
+      return NULL;
+    }
+    memcpy(p[i],src[i],addr_len);
+  }
+  p[ar_len] = NULL;
+  return p;
+}
+#endif
+#if !defined(HAVE_NETDB_REENTRANT) || defined(NON_R_GETHOSTBYADDR) || defined(NON_R_GETHOSTBYNAME)
+static char **
+c_copy_string_array(char **src)
+{
+  char ** p = src;
+  size_t i = 0 ;
+  size_t len ;
+  if ( src == NULL ){
+    return NULL;
+  }
+  while ( *p ){
+    i++;
+    p++;
+  }
+  len = i;
+  p = malloc((len+1) * sizeof(char *));
+  if ( p == NULL ){
+    return NULL;
+  }
+  for ( i = 0 ; i < len ; ++i ){
+    p[i] = strdup(src[i]);
+    if ( p[i] == NULL ){
+      size_t j;
+      for ( j = 0 ; j < i ; j++ ){
+        free(p[j]);
+      }
+      free(p);
+      return NULL;
+    }
+  }
+  p[len] = NULL;
+  return p;
+}
+
+static void c_free_string_array(char ** src)
+{
+  if ( src ){
+    char ** p = src;
+    while (*p){
+      free(*p);
+      ++p;
+    }
+    free(src);
+  }
+}
+
+static inline char * s_strdup (const char *s){
+  return (strdup( s == NULL ? "" : s ));
+}
+#endif
+
 /* +-----------------------------------------------------------------+
    | JOB: gethostname                                                |
    +-----------------------------------------------------------------+ */
@@ -1676,7 +1935,7 @@ static value result_gethostname(struct job_gethostname *job)
   return result;
 }
 
-CAMLprim value lwt_unix_gethostname_job()
+CAMLprim value lwt_unix_gethostname_job(value Unit)
 {
   LWT_UNIX_INIT_JOB(job, gethostname, 0);
   return lwt_unix_alloc_job(&(job->job));
@@ -1692,7 +1951,9 @@ struct job_gethostbyname {
   struct lwt_unix_job job;
   struct hostent entry;
   struct hostent *ptr;
+#ifndef NON_R_GETHOSTBYNAME
   char buffer[NETDB_BUFFER_SIZE];
+#endif
   char *name;
   char data[];
 };
@@ -1748,16 +2009,80 @@ static value alloc_host_entry(struct hostent *entry)
   return res;
 }
 
+#if defined(NON_R_GETHOSTBYADDR) || defined(NON_R_GETHOSTBYNAME)
+static struct hostent *
+hostent_dup(struct hostent *orig)
+{
+  if ( orig == NULL ){
+    return NULL;
+  }
+  struct hostent *h = malloc(sizeof *h);
+  if ( h == NULL ){
+    return NULL;
+  }
+  h->h_name = s_strdup(orig->h_name);
+  if ( !h->h_name ){
+    goto nomem1;
+  }
+  if ( !orig->h_aliases ){
+    h->h_aliases = NULL;
+  }
+  else {
+    h->h_aliases = c_copy_string_array(orig->h_aliases);
+    if ( !h->h_aliases){
+      goto nomem2;
+    }
+  }
+  if ( !orig->h_addr_list ){
+    h->h_addr_list = NULL;
+  }
+  else {
+    h->h_addr_list = c_copy_addr_array(orig->h_addr_list,orig->h_length);
+    if ( !h->h_addr_list ){
+      goto nomem3;
+    }
+  }
+  h->h_addrtype = orig->h_addrtype;
+  h->h_length = orig->h_length;
+  return h;
+nomem3:
+  c_free_string_array(h->h_aliases);
+nomem2:
+  free(h->h_name);
+nomem1:
+  free(h);
+  return NULL;
+}
+
+static void
+hostent_free(struct hostent *h)
+{
+  if ( h ){
+    c_free_string_array(h->h_addr_list);
+    c_free_string_array(h->h_aliases);
+    free(h->h_name);
+    free(h);
+  }
+}
+#endif
+
 static void worker_gethostbyname(struct job_gethostbyname *job)
 {
-  int h_errno;
 #if HAS_GETHOSTBYNAME_R == 5
+  int h_errno;
   job->ptr = gethostbyname_r(job->name, &job->entry, job->buffer, NETDB_BUFFER_SIZE, &h_errno);
 #elif HAS_GETHOSTBYNAME_R == 6
+  int h_errno;
   if (gethostbyname_r(job->name, &job->entry, job->buffer, NETDB_BUFFER_SIZE, &(job->ptr), &h_errno) != 0)
     job->ptr = NULL;
 #else
-  job->ptr = NULL;
+  job->ptr = gethostbyname(job->name);
+  if (job->ptr) {
+    job->ptr= hostent_dup(job->ptr);
+    if ( job->ptr ){
+      job->entry = *job->ptr;
+    }
+  }
 #endif
 }
 
@@ -1768,6 +2093,9 @@ static value result_gethostbyname(struct job_gethostbyname *job)
     caml_raise_not_found();
   } else {
     value entry = alloc_host_entry(&job->entry);
+#ifdef NON_R_GETHOSTBYNAME
+    hostent_free(job->ptr);
+#endif
     lwt_unix_free_job(&job->job);
     return entry;
   }
@@ -1788,19 +2116,28 @@ struct job_gethostbyaddr {
   struct in_addr addr;
   struct hostent entry;
   struct hostent *ptr;
+#ifndef NON_R_GETHOSTBYADDR
   char buffer[NETDB_BUFFER_SIZE];
+#endif
 };
 
 static void worker_gethostbyaddr(struct job_gethostbyaddr *job)
 {
-  int h_errno;
 #if HAS_GETHOSTBYADDR_R == 7
+  int h_errno;
   job->ptr = gethostbyaddr_r(&job->addr, 4, AF_INET, &job->entry, job->buffer, NETDB_BUFFER_SIZE, &h_errno);
 #elif HAS_GETHOSTBYADDR_R == 8
+  int h_errno;
   if (gethostbyaddr_r(&job->addr, 4, AF_INET, &job->entry, job->buffer, NETDB_BUFFER_SIZE, &job->ptr, &h_errno) != 0)
     job->ptr = NULL;
 #else
-  job->ptr = NULL;
+  job->ptr = gethostbyaddr(&job->addr, 4, AF_INET);
+  if (job->ptr) {
+    job->ptr = hostent_dup(job->ptr);
+    if ( job->ptr ){
+      job->entry = *job->ptr;
+    }
+  }
 #endif
 }
 
@@ -1811,6 +2148,9 @@ static value result_gethostbyaddr(struct job_gethostbyaddr *job)
     caml_raise_not_found();
   } else {
     value entry = alloc_host_entry(&job->entry);
+#ifdef NON_R_GETHOSTBYADDR
+    hostent_free(job->ptr);
+#endif
     lwt_unix_free_job(&job->job);
     return entry;
   }
@@ -1889,7 +2229,6 @@ static value alloc_servent(struct servent *entry)
         break;                                                          \
       case ENOENT:                                                      \
       default:                                                          \
-        free(job->buffer);                                              \
         job->ptr = NULL;                                                \
         return;                                                         \
       }                                                                 \
@@ -1918,6 +2257,86 @@ static value alloc_servent(struct servent *entry)
 
 #else /* defined(HAVE_NETDB_REENTRANT) */
 
+static struct servent * servent_dup(const struct servent * serv)
+{
+  struct servent * s;
+  if (!serv){
+    return NULL;
+  }
+  s = malloc(sizeof *s);
+  if ( s == NULL ){
+    goto nomem1;
+  }
+  s->s_name = s_strdup(serv->s_name);
+  if ( s->s_name == NULL ){
+    goto nomem2;
+  }
+  s->s_proto = s_strdup(serv->s_proto);
+  if ( s->s_proto == NULL ){
+    goto nomem3;
+  }
+  s->s_aliases = c_copy_string_array(serv->s_aliases);
+  if ( s->s_aliases == NULL && serv->s_aliases != NULL ){
+    goto nomem4;
+  }
+  s->s_port = serv->s_port;
+  return s;
+nomem4:
+  free(s->s_proto);
+nomem3:
+  free(s->s_name);
+nomem2:
+  free(s);
+nomem1:
+  return NULL;
+}
+
+static void servent_free(struct servent * s)
+{
+  if ( ! s ){
+    return;
+  }
+  free(s->s_proto);
+  free(s->s_name);
+  c_free_string_array(s->s_aliases);
+  free(s);
+}
+
+static struct protoent * protoent_dup(const struct protoent * proto)
+{
+  if (!proto){
+    return NULL;
+  }
+  struct protoent * p = malloc(sizeof *p);
+  if ( p == NULL ){
+    return NULL;
+  }
+  p->p_name = s_strdup(proto->p_name);
+  if ( p->p_name == NULL ){
+    goto nomem1;
+  }
+  p->p_aliases = c_copy_string_array( proto->p_aliases );
+  if ( p->p_aliases == NULL && proto->p_aliases != NULL ){
+    goto nomem2;
+  }
+  p->p_proto = proto->p_proto;
+  return p;
+nomem2:
+  free(p->p_name);
+nomem1:
+  free(p);
+  return NULL;
+}
+
+static void protoent_free(struct protoent * p)
+{
+  if ( p ){
+    free(p->p_name);
+    c_free_string_array(p->p_aliases);
+    free(p);
+  }
+}
+
 #define JOB_GET_ENTRY2(INIT, FUNC, TYPE, ARGS_VAL, ARGS_DECL, ARGS_CALL) \
   struct job_##FUNC {                                                   \
     struct lwt_unix_job job;                                            \
@@ -1928,6 +2347,11 @@ static value alloc_servent(struct servent *entry)
   static void worker_##FUNC(struct job_##FUNC *job)                     \
   {                                                                     \
     job->entry = FUNC(ARGS_CALL);                                       \
+    if ( job->entry ){                                                  \
+      job->entry = TYPE ## _dup ( job->entry );                         \
+      if (! job->entry ){                                               \
+      }                                                                 \
+    }                                                                   \
   }                                                                     \
                                                                         \
   static value result_##FUNC(struct job_##FUNC *job)                    \
@@ -1937,6 +2361,7 @@ static value alloc_servent(struct servent *entry)
       caml_raise_not_found();                                           \
     } else {                                                            \
       value res = alloc_##TYPE(job->entry);                             \
+      TYPE ## _free ( job->entry );                                     \
       lwt_unix_free_job(&job->job);                                     \
       return res;                                                       \
     }                                                                   \
@@ -1974,7 +2399,7 @@ JOB_GET_ENTRY2(LWT_UNIX_INIT_JOB_STRING2(job, getservbyname, 0, name, proto),
                char data[],
                ARGS(job->name, job->proto))
 JOB_GET_ENTRY2(LWT_UNIX_INIT_JOB_STRING(job, getservbyport, 0, proto);
-               job->port = Int_val(port),
+               job->port = htons(Int_val(port)),
                getservbyport,
                servent,
                ARGS(value port, value proto),
@@ -2048,7 +2473,8 @@ static value result_getaddrinfo(struct job_getaddrinfo *job)
       vres = v;
     }
   }
-  freeaddrinfo(job->info);
+  if (job->info != NULL)
+    freeaddrinfo(job->info);
   lwt_unix_free_job(&job->job);
   CAMLreturn(vres);
 }
@@ -2435,3 +2861,14 @@ CAMLprim value lwt_unix_tcsetattr_job(value fd, value when, value termios)
   memcpy(&job->termios, &Field(termios, 0), NFIELDS * sizeof(value));
   return lwt_unix_alloc_job(&job->job);
 }
+
+/* +-----------------------------------------------------------------+
+   | Unavailable primitives                                          |
+   +-----------------------------------------------------------------+ */
+
+LWT_NOT_AVAILABLE1(unix_is_socket)
+LWT_NOT_AVAILABLE1(unix_socketpair_stub)
+LWT_NOT_AVAILABLE1(unix_system_job)
+LWT_NOT_AVAILABLE4(process_create_process)
+LWT_NOT_AVAILABLE1(process_wait_job)
+LWT_NOT_AVAILABLE2(process_terminate_process)
