@@ -1,4 +1,4 @@
-/* Lightweight thread library for Objective Caml
+/* Lightweight thread library for OCaml
  * http://www.ocsigen.org/lwt
  * Module Lwt_unix_unix
  * Copyright (C) 2010 Jérémie Dimino
@@ -163,7 +163,7 @@ CAMLprim value lwt_unix_bytes_read(value fd, value buf, value vofs, value vlen)
    | Memory mapped files                                             |
    +-----------------------------------------------------------------+ */
 
-CAMLprim value lwt_unix_get_page_size()
+CAMLprim value lwt_unix_get_page_size(value Unit)
 {
   SYSTEM_INFO si;
   GetSystemInfo(&si);
@@ -505,3 +505,67 @@ CAMLprim value lwt_unix_system_job(value cmdline)
     return lwt_unix_alloc_job(&(job->job));
   }
 }
+
+/* +-----------------------------------------------------------------+
+   | Unavailable primitives                                          |
+   +-----------------------------------------------------------------+ */
+
+LWT_NOT_AVAILABLE2(unix_mcast_set_loop)
+LWT_NOT_AVAILABLE2(unix_mcast_set_ttl)
+LWT_NOT_AVAILABLE4(unix_mcast_modify_membership)
+LWT_NOT_AVAILABLE1(unix_get_credentials)
+LWT_NOT_AVAILABLE1(unix_get_cpu)
+LWT_NOT_AVAILABLE1(unix_get_affinity)
+LWT_NOT_AVAILABLE2(unix_set_affinity)
+LWT_NOT_AVAILABLE5(unix_bytes_recv)
+LWT_NOT_AVAILABLE5(unix_bytes_send)
+LWT_NOT_AVAILABLE3(unix_bytes_recv_msg)
+LWT_NOT_AVAILABLE3(unix_bytes_send_msg)
+LWT_NOT_AVAILABLE6(unix_bytes_sendto)
+LWT_NOT_AVAILABLE6(unix_bytes_sendto_byte)
+LWT_NOT_AVAILABLE6(unix_madvise)
+LWT_NOT_AVAILABLE4(unix_mincore)
+LWT_NOT_AVAILABLE2(unix_wait_mincore_job)
+LWT_NOT_AVAILABLE1(unix_guess_blocking_job)
+LWT_NOT_AVAILABLE1(unix_readable)
+LWT_NOT_AVAILABLE1(unix_writable)
+LWT_NOT_AVAILABLE1(unix_open_job)
+LWT_NOT_AVAILABLE1(unix_stat_job)
+LWT_NOT_AVAILABLE1(unix_lstat_job)
+LWT_NOT_AVAILABLE1(unix_fstat_job)
+LWT_NOT_AVAILABLE1(unix_isatty_job)
+LWT_NOT_AVAILABLE1(unix_stat_64_job)
+LWT_NOT_AVAILABLE1(unix_lstat_64_job)
+LWT_NOT_AVAILABLE1(unix_fstat_64_job)
+LWT_NOT_AVAILABLE1(unix_opendir_job)
+LWT_NOT_AVAILABLE1(unix_readdir_job)
+LWT_NOT_AVAILABLE2(unix_readdir_n_job)
+LWT_NOT_AVAILABLE1(unix_rewinddir_job)
+LWT_NOT_AVAILABLE1(unix_closedir_job)
+LWT_NOT_AVAILABLE1(unix_readlink_job)
+LWT_NOT_AVAILABLE3(unix_lockf_job)
+LWT_NOT_AVAILABLE5(unix_recv)
+LWT_NOT_AVAILABLE5(unix_send)
+LWT_NOT_AVAILABLE5(unix_recvfrom)
+LWT_NOT_AVAILABLE6(unix_sendto)
+LWT_NOT_AVAILABLE6(unix_sendto_byte)
+LWT_NOT_AVAILABLE3(unix_recv_msg)
+LWT_NOT_AVAILABLE3(unix_send_msg)
+LWT_NOT_AVAILABLE1(unix_gethostname_job)
+LWT_NOT_AVAILABLE1(unix_gethostbyname_job)
+LWT_NOT_AVAILABLE1(unix_gethostbyaddr_job)
+LWT_NOT_AVAILABLE1(unix_getprotobyname_job)
+LWT_NOT_AVAILABLE1(unix_getprotobynumber_job)
+LWT_NOT_AVAILABLE2(unix_getservbyname_job)
+LWT_NOT_AVAILABLE2(unix_getservbyport_job)
+LWT_NOT_AVAILABLE3(unix_getaddrinfo_job)
+LWT_NOT_AVAILABLE2(unix_getnameinfo_job)
+LWT_NOT_AVAILABLE1(unix_tcgetattr_job)
+LWT_NOT_AVAILABLE3(unix_tcsetattr_job)
+LWT_NOT_AVAILABLE1(unix_getlogin_job)
+LWT_NOT_AVAILABLE1(unix_getpwnam_job)
+LWT_NOT_AVAILABLE1(unix_getgrnam_job)
+LWT_NOT_AVAILABLE1(unix_getpwuid_job)
+LWT_NOT_AVAILABLE1(unix_getgrgid_job)
+LWT_NOT_AVAILABLE4(unix_wait4)
+LWT_NOT_AVAILABLE5(unix_bytes_recvfrom)
