@@ -1,4 +1,4 @@
-(* Lightweight thread library for Objective Caml
+(* Lightweight thread library for OCaml
  * http://www.ocsigen.org/lwt
  * Module Pa_lwt
  * Copyright (C) 2009 Jérémie Dimino
@@ -164,7 +164,7 @@
         done
       ]}
 
-    - pattern mattching:
+    - pattern matching:
 
       {[
         match_lwt <expr> with
