@@ -1,4 +1,4 @@
-(* Lightweight thread library for Objective Caml
+(* Lightweight thread library for OCaml
  * http://www.ocsigen.org/lwt
  * Module Main
  * Copyright (C) 2009 Jérémie Dimino
@@ -23,5 +23,5 @@
 Test.run "core" [
   Test_lwt.suite;
   Test_lwt_stream.suite;
-  Test_lwt_util.suite;
+  Test_lwt_list.suite;
 ]
