@@ -1,4 +1,4 @@
-(* Lightweight thread library for Objective Caml
+(* Lightweight thread library for OCaml
  * http://www.ocsigen.org/lwt
  * Module Main
  * Copyright (C) 2009 Jérémie Dimino
@@ -23,4 +23,5 @@
 Test.run "unix" [
   Test_lwt_io.suite;
   Test_lwt_io_non_block.suite;
+  Test_mcast.suite;
 ]
